subroutine moments(par_n, par, mom_n, mom)

use globvar
use esolver
use etoolbox
use csv_file
implicit none

integer, intent(in) :: par_n
real(8), intent(in) :: par(par_n)
integer, intent(in) :: mom_n
real(8), intent(out) :: mom(mom_n)

real(8), parameter :: toll = 1.0E-16

! outcome variables (aggregate)
! unconditional on ptn
real(8) :: noequ, numequ, conequ                        ! no pure equilibria, num of equilibria, consistent equilibria
real(8) :: upsg(3), uini(3), ueff(3)                    ! average passage rate - after/before lobbying (all/actual passage/actual failure) 
real(8) :: uspd(nr), uptn(nr), uiptn(nr), urptn(nr)     ! total spending, average ptn, irrelevant ptn, relevant ptn
real(8) :: uvalm(nr), uvalv(nr)                         ! mean and variance of value of policy
real(8) :: utrtn(nr), uartn(nr)                         ! avg return during the period, avg return per policy
real(8) :: uwelg(nr)                                    ! welfare gain of lobbying ban
real(8) :: ueffc(nr)                                    ! effect of lobbying of each player
real(8) :: uentry(nr,2)                                 ! entry cost
real(8) :: ucorr(nr,nr)                                 ! ptn correlation
real(8) :: ueffs, ueffa
! conditional on ptn
real(8) :: cpsg(3), cini(3), ceff(3)                    ! average passage rate - after/before lobbying (all/actual passage/actual failure) 
real(8) :: cspd(nr), cvalm(nr), cvalv(nr)               ! total spending, mean and variance of value of policy
real(8) :: ctrtn(nr), cartn(nr)                         ! avg return during the period, avg return per policy
real(8) :: cwelg(nr)                                    ! welfare gain of lobbying ban 
real(8) :: ceffc(nr)                                    ! effect of lobbying of each player
real(8) :: ceffs, ceffa
! conditional on categories
real(8) :: cat1_psg(ncat1), cat2_psg(ncat2)    		! average passage rate conditional on categories
real(8) :: cat1_ini(ncat1), cat2_ini(ncat2)    		! average initial probl conditional on categories
real(8) :: cat1_eff(ncat1), cat2_eff(ncat2)    		! average effect of lobbying conditional on categories
real(8) :: cat1_ptn(ncat1,nr), cat2_ptn(ncat2,nr)   ! average participation rate conditional on categories
real(8) :: cat2_effs(ncat2), cat2_effa(ncat2), cat2_can(ncat2)

! data summary
real(8) :: rst_aptn(nr), rst_iptn(nr), rst_rptn(nr)     ! data summary stats - ptn (average, irelevant, relevant)
real(8) :: rst_apsg, renum(nr), ienum(nr)
real(8) :: rst_c1psg(ncat1), rst_c2psg(ncat2)
real(8) :: rst_c1ptn(ncat1,nr), rst_c2ptn(ncat2,nr) 
real(8) :: rst_corr(nr,nr)

! list of policies by the effect
real(8) :: listeff(np,13)

! individual variables
real(8) :: sim_ini(np,ns), sim_fin(np,ns)
real(8) :: sim_col(np,ns), sim_oil(np,ns), sim_nuc(np,ns),sim_ren(np,ns)
real(8) :: sim_neq(np,ns)

! bill-level variables
real(8) :: b_inin(nl,ns), b_inim(nl,ns), b_inia(nl,ns) 
real(8) :: b_finn(nl,ns), b_finm(nl,ns), b_fina(nl,ns)
real(8) :: b_effn(nl,ns), b_effm(nl,ns), b_effa(nl,ns)
real(8) :: b_col(nl,ns), b_oil(nl,ns), b_nuc(nl,ns), b_ren(nl,ns)
real(8) :: b_neq(nl,ns)
real(8) :: bdem(nl)
integer :: bpsg(nl)

! varibles for calculation
real(8) :: iz, thes, util1, util0, chk, iniprob
integer :: gg, ss, ii, jj, kk, cc, enum, dev, jjn, mdx, equinx(num), rn, in, bb
real(8) :: v(nr), equprob(num), payoff(num,nr), spend(num,nr), equsum(num), diff(num,nr)
real(8) :: numlob(3), nck(3)

real(8) :: no_equ, con_equ, pck
real(8) :: u_psg, u_ini, u_eff, u_ptn(nr), u_rptn(nr), u_iptn(nr), u_spd(nr), u_trtn(nr)
real(8) :: u_artn(nr), u_welg(nr), u_valm(nr), u_valv(nr), u_entry(nr,2), u_effs, u_effa
real(8) :: c_psg, c_ini, c_eff, c_ref, c_spd(nr), c_trtn(nr), c_artn(nr), c_welg(nr)
real(8) :: c_valm(nr), c_valv(nr), u_effc(nr), c_effc(nr), c_effs, c_effa, c_can  
real(8) :: alt_thn1, alt_thn2, alt_prob, fspd, aspd, u_corr(nr,nr), stdev_ptn(nr), ef1, ef2
integer :: j1, j2, bid, pid, pnm



	! (0) Parameters
	
    call parscale_avar(par_n,par)
    
    pck = 0.0d0  
	if (bf.le.0.0d0) then
        pck = 1.0d0
        bf = toll
    endif
	if (ba.le.0.0d0) then
        pck = 1.0d0
        ba = toll
    endif
    if (gam.le.0.0d0) then
        pck = 1.0d0
        gam = toll
    endif
    if (gam.ge.1.0d0) then
        pck = 1.0d0
        gam = 1.0d0-toll
    endif
    if (varin.le.0.0d0) then
        pck = 1.0d0
        varin = 0.0d0
    endif
	do jj = 1,nr
	    if (varvu(jj).le.0.0d0) then
            pck = 1.0d0
	        varvu(jj) = 0.0d0
	    end if
	end do
	
	! (1) initialize variables

    do ii = 1,mom_n
        mom(ii) = 0.0d0
    end do

    noequ = 0.0d0
    numequ = 0.0d0
    conequ = 0.0d0
    do jj = 1,3
        upsg(jj) = 0.0d0
        uini(jj) = 0.0d0
        ueff(jj) = 0.0d0
        cpsg(jj) = 0.0d0
        cini(jj) = 0.0d0
        ceff(jj) = 0.0d0
    end do
    do ii = 1,ncat1
        cat1_psg(ii) = 0.0d0
        cat1_ini(ii) = 0.0d0
        cat1_eff(ii) = 0.0d0        
        do jj = 1,nr
            cat1_ptn(ii,jj) = 0.0d0
        end do     
    end do
    do ii = 1,ncat2
        cat2_psg(ii) = 0.0d0
        cat2_ini(ii) = 0.0d0
        cat2_eff(ii) = 0.0d0
        cat2_effs(ii) = 0.0d0
        cat2_effa(ii) = 0.0d0
        cat2_can(ii) = 0.0d0
        do jj = 1,nr
            cat2_ptn(ii,jj) = 0.0d0
        end do   
    end do  

    do jj = 1,nr
        uspd(jj) = 0.0d0
        uptn(jj) = 0.0d0
        uiptn(jj) = 0.0d0
        urptn(jj) = 0.0d0
        uvalm(jj) = 0.0d0
        uvalv(jj) = 0.0d0
        utrtn(jj) = 0.0d0
        uartn(jj) = 0.0d0
        uwelg(jj) = 0.0d0
        ueffc(jj) = 0.0d0
        uentry(jj,1) = 0.0d0
        uentry(jj,2) = 0.0d0
        cspd(jj) = 0.0d0
        cvalm(jj) = 0.0d0
        cvalv(jj) = 0.0d0
        ctrtn(jj) = 0.0d0
        cartn(jj) = 0.0d0
        cwelg(jj) = 0.0d0
        ceffc(jj) = 0.0d0
    end do 
    
    do j1= 1,nr
        do j2 = 1,nr
            ucorr(j1,j2) = 0.0d0
        end do
    end do
    
    ueffs = 0.0d0
    ueffa = 0.0d0
    ceffs = 0.0d0
    ceffa = 0.0d0    
    
    numlob(1) = 0.0d0
    numlob(2) = 0.0d0
    numlob(3) = 0.0d0
    do gg = 1,np
        nck(1) = 0.0d0
        nck(2) = 0.0d0
        nck(3) = 0.0d0
        do jj = 1,nr
            if (rst_ptn(gg,jj) == 1) then
                nck(1) = 1.0d0
                if (W(gg,jj) == 1) then
                    nck(2) = 1.0d0
                else
                    nck(3) = 1.0d0
                end if
             end if
        end do
        numlob(1) = numlob(1) + nck(1)
        numlob(2) = numlob(2) + nck(2)
        numlob(3) = numlob(3) + nck(3)
    end do

    do gg = 1,np
       do ss = 1,ns
	   sim_ini(gg,ss) = 0.0d0
	   sim_fin(gg,ss) = 0.0d0
	   sim_col(gg,ss) = 0.0d0
	   sim_oil(gg,ss) = 0.0d0
	   sim_nuc(gg,ss) = 0.0d0
	   sim_ren(gg,ss) = 0.0d0
           sim_neq(gg,ss) = 1.0d0
       end do
    end do 
    
!-------------------------------------------------------------
! (2) Equilibrium for each case
!-------------------------------------------------------------

    do gg = 1,np 
        u_psg = 0.0d0
        u_ini = 0.0d0
        u_eff = 0.0d0
        c_psg = 0.0d0
        c_ini = 0.0d0
        c_eff = 0.0d0
        c_ref = 0.0d0     
        do jj = 1,nr
            u_ptn(jj) = 0.0d0
            u_iptn(jj) = 0.0d0
            u_rptn(jj) = 0.0d0
            u_spd(jj) = 0.0d0
            u_trtn(jj) = 0.0d0
            u_artn(jj) = 0.0d0
            u_welg(jj) = 0.0d0
            u_valm(jj) = 0.0d0
            u_valv(jj) = 0.0d0
            u_effc(jj) = 0.0d0
            c_spd(jj) = 0.0d0
            c_trtn(jj) = 0.0d0
            c_artn(jj) = 0.0d0
            c_welg(jj) = 0.0d0
            c_valm(jj) = 0.0d0
            c_valv(jj) = 0.0d0  
            c_effc(jj) = 0.0d0          
        end do
        do jj = 1,nr
            u_entry(jj,1) = ect+cect*L(gg,jj)
            u_entry(jj,2) = (ect+cect*L(gg,jj))**2.0d0
        end do
        do j1= 1,nr
            do j2 = 1,nr
                u_corr(j1,j2) = 0.0d0
            end do
        end do
        
        u_effs = 0.0d0
        u_effa = 0.0d0
        c_effs = 0.0d0
        c_effa = 0.0d0
        c_can = 0.0d0
        
        no_equ = 0.0d0
        con_equ = 0.0d0        
	do ss = 1,ns
	    kk = ns*(gg-1) + ss
	    !(2-1) Generate theta's
	    iz = 0.0
	    do ii = 1,6
	        iz = iz + Z(gg,ii)*del(ii)
	    end do
	    thes = iz+varin*simshock(kk,1)
	    iniprob = al_norm(thes,.False.)

	    !(2-2) Generate values
	    do jj = 1,nr
			if (W(gg,jj).eq.1) then
				v(jj) =exp(alp(jj)+alp(jj+nr)*X(gg,jj)+simshock(kk,(jj+1))*varvu(jj))
			else
				v(jj) =-exp(alp(jj)+alp(jj+nr)*X(gg,jj)+simshock(kk,(jj+1))*varvu(jj))
			end if
	    end do

	    !(2-3) Calculate equilibrium in each participation case

		do ii = 1,num
			call sim_esearch(v,ptn(ii,:),thes,L(gg,:),equprob(ii),payoff(ii,:),spend(ii,:))
		end do

		!(2-4) Find equilibria by checking a deviation
		enum = 0
		do ii = 1,num
		    equinx(ii) = 0
            jj = 1
            dev = 0
		    do while((jj<=nr).and.(dev.eq.0))
		        jjn = int(2.0d0**((nr-jj)*1.0d0))
			    util0 = payoff(ii,jj)			    
		        if (ptn(ii,jj)==0) then
		            util1 = payoff((ii+jjn),jj)
		        else
		            util1 = payoff((ii-jjn),jj)
		        end if
		        diff(ii,jj) = util0-util1
			    if (util1>util0) then
			        dev = 1
		        end if
			    jj = jj+1
		     end do
		     if (dev.eq.0) then
                 enum = enum+1
                equinx(enum) = ii
                equsum(enum) = 0.0d0
                do jj = 1,nr
                    equsum(enum) = equsum(enum)+payoff(ii,jj)
                end do                    
		     end if
        end do

		!(2-5) Find an equilibrium that is pareto optimal and check if the equilibrium is consistent
		!    with the observed entry profile.
		    
		if (enum.eq.0) then
            sim_neq(gg,ss) = 0.0d0	
            sim_ini(gg,ss) = 0.0d0
            sim_fin(gg,ss) = 0.0d0
            sim_col(gg,ss) = v(1)
            sim_oil(gg,ss) = v(2)
            sim_nuc(gg,ss) = v(3)
            sim_ren(gg,ss) = v(4)	
		else
            mdx = equinx(maxloc(equsum(1:enum),1))          ! mdx: index number of the selected equilibrium
            sim_ini(gg,ss) = iniprob
            sim_fin(gg,ss) = equprob(mdx)
            sim_col(gg,ss) = v(1)
            sim_oil(gg,ss) = v(2)
            sim_nuc(gg,ss) = v(3)
            sim_ren(gg,ss) = v(4)
		end if													 
	end do
   end do

   ! (3) aggregate over bills

   do bb = 1,nl
       do ss = 1,ns
           b_inin(bb,ss) = 1.0d0
           b_inim(bb,ss) = 0.0d0
           b_inia(bb,ss) = 0.0d0
           b_finn(bb,ss) = 1.0d0
           b_finm(bb,ss) = 0.0d0
           b_fina(bb,ss) = 0.0d0
           b_effn(bb,ss) = 1.0d0
           b_effm(bb,ss) = 0.0d0
           b_effa(bb,ss) = 0.0d0
           b_col(bb,ss) = 0.0d0
           b_oil(bb,ss) = 0.0d0
           b_nuc(bb,ss) = 0.0d0
           b_ren(bb,ss) = 0.0d0
           b_neq(bb,ss) = 1.0d0
       end do
   end do


   do ss = 1,ns
       do kk = 1,nb
           bid = bp(kk,1)
           pid = bp(kk,2)
           pnm = bp(kk,3)
           if (sim_neq(pid,ss).eq. 1.0d0) then
                b_inin(bid,ss) = min(b_inin(bid,ss),sim_ini(pid,ss))
                b_inim(bid,ss) = max(b_inim(bid,ss),sim_ini(pid,ss))
                b_inia(bid,ss) = b_inia(bid,ss)+sim_ini(pid,ss)/(pnm*1.0d0)
                b_finn(bid,ss) = min(b_finn(bid,ss),sim_fin(pid,ss))
                b_finm(bid,ss) = max(b_finm(bid,ss),sim_fin(pid,ss))
                b_fina(bid,ss) = b_fina(bid,ss)+sim_fin(pid,ss)/(pnm*1.0d0)
                b_effn(bid,ss) = min(b_effn(bid,ss),abs(sim_ini(pid,ss)-sim_fin(pid,ss)))
                b_effm(bid,ss) = max(b_effm(bid,ss),abs(sim_ini(pid,ss)-sim_fin(pid,ss)))
                b_effa(bid,ss) = b_effa(bid,ss)+abs(sim_ini(pid,ss)-sim_fin(pid,ss))/(pnm*1.0d0)
           else
                b_neq(bid,ss) = 0.0d0
           end if
           b_col(bid,ss) = b_col(bid,ss)+sim_col(pid,ss)
           b_oil(bid,ss) = b_oil(bid,ss)+sim_oil(pid,ss)
           b_nuc(bid,ss) = b_nuc(bid,ss)+sim_nuc(pid,ss)
           b_ren(bid,ss) = b_ren(bid,ss)+sim_ren(pid,ss)                
       end do
   end do 
   
   do bb = 1,nl
       do ss = 1,ns
           b_col(bb,ss) = abs(b_col(bb,ss))
           b_oil(bb,ss) = abs(b_oil(bb,ss))
           b_nuc(bb,ss) = abs(b_nuc(bb,ss))
           b_ren(bb,ss) = abs(b_ren(bb,ss))
       end do
   end do 

	
   ! (5) matrix
    
   do bb = 1,nl
       bpsg(bb) = 0
   end do
   bpsg(1) = 1
   bpsg(39) = 1
   bpsg(83) = 1
   bpsg(168) = 1
   bpsg(296) = 1

   do ii = 1,mom_n
      mom(ii) = 0.0d0
   end do 
   do bb = 1,nl
       bdem(bb) = 0.0d0
       do ss = 1,ns
           bdem(bb) = bdem(bb)+b_neq(bb,ss)
       end do
       do ss = 1,ns
           mom(1) = mom(1)+b_inin(bb,ss)*b_neq(bb,ss)/(bdem(bb)*(nl*1.0d0))
           mom(2) = mom(2)+b_inim(bb,ss)*b_neq(bb,ss)/(bdem(bb)*(nl*1.0d0))
           mom(3) = mom(3)+b_inia(bb,ss)*b_neq(bb,ss)/(bdem(bb)*(nl*1.0d0))
           mom(4) = mom(4)+b_finn(bb,ss)*b_neq(bb,ss)/(bdem(bb)*(nl*1.0d0))
           mom(5) = mom(5)+b_finm(bb,ss)*b_neq(bb,ss)/(bdem(bb)*(nl*1.0d0))
           mom(6) = mom(6)+b_fina(bb,ss)*b_neq(bb,ss)/(bdem(bb)*(nl*1.0d0))
           mom(7) = mom(7)+b_effn(bb,ss)*b_neq(bb,ss)/(bdem(bb)*(nl*1.0d0))
           mom(8) = mom(8)+b_effm(bb,ss)*b_neq(bb,ss)/(bdem(bb)*(nl*1.0d0))
           mom(9) = mom(9)+b_effa(bb,ss)*b_neq(bb,ss)/(bdem(bb)*(nl*1.0d0))
           mom(10) = mom(10)+b_col(bb,ss)/(ns*nl*1.0d0)
           mom(11) = mom(11)+b_oil(bb,ss)/(ns*nl*1.0d0)
           mom(12) = mom(12)+b_nuc(bb,ss)/(ns*nl*1.0d0)
           mom(13) = mom(13)+b_ren(bb,ss)/(ns*nl*1.0d0)
           if (bpsg(bb) == 1) then
               mom(14) = mom(14)+b_effa(bb,ss)*b_neq(bb,ss)/(bdem(bb)*(5*1.0d0))
               mom(16) = mom(16)+b_effm(bb,ss)*b_neq(bb,ss)/(bdem(bb)*(5*1.0d0))
           else
               mom(15) = mom(15)+b_effa(bb,ss)*b_neq(bb,ss)/(bdem(bb)*((nl-5)*1.0d0))
               mom(17) = mom(17)+b_effm(bb,ss)*b_neq(bb,ss)/(bdem(bb)*((nl-5)*1.0d0))
           end if
       end do
   end do
   mom(18) = pck
                   
end subroutine moments