subroutine mvalue(par_n, par, gm, lval, lspd)

! This code calculates likelihood for each observation.
use globvar
use etoolbox
use esolver

implicit none

integer, intent(in) :: par_n
real(8), intent(in) :: par(par_n)
integer, intent(in) :: gm
real(8), intent(out) :: lval
real(8), intent(out) :: lspd(nr)

real(8), parameter :: toll = 1.0E-16

integer :: ss, kk, ii, jj, enum, jjn, dev, mdx, pck
real(8) :: iz, thes, lcon, util0, util1, lck 

real(8), dimension(nr) :: v
integer, dimension(num) :: equinx
real(8), dimension(num):: equthn, equsum
real(8), dimension(num,nr) :: spend, payoff

	call parscale_avar(par_n, par)
	
	lval = 0.0d0
	lcon = 0.0d0
	do jj = 1,nr
    	    lspd(jj) = 0.0d0
	end do

    pck = 0  
	if (bf.le.0.0d0) then
        pck = 1
        bf = toll
    endif
	if (ba.le.0.0d0) then
        pck = 1
        ba = toll
    endif
    if (gam.le.0.0d0) then
        pck = 1
        gam = toll
    endif
    if (gam.ge.1.0d0) then
        pck = 1
        gam = 1.0d0-toll
    endif
    if (varin.le.0.0d0) then
        pck = 1
        varin = 0.0d0
    endif
	do jj = 1,nr
	    if (varvu(jj).le.0.0d0) then
            pck = 1
	        varvu(jj) = 0.0d0
	    end if
	end do
	
	if (pck == 1) then
	    write(1000,*) "Parameter went out of bounds"
	end if

	do ss = 1,ns
	    kk = (gm-1)*ns+ss
	    !(1) Generate theta's
	    iz = 0.0d0
	    do ii = 1,7
	        iz = iz + Z(gm,ii)*del(ii)
	    end do			    
	    thes = iz+varin*simshock(kk,1)		    
	    !(2) Generate values
	    do jj = 1,nr
			      if (W(gm,jj).eq.1) then
				    v(jj) =exp(alp(jj)+simshock(kk,(jj+1))*varvu(jj))*(X(gm,5)-X(gm,jj))	
				    v(jj) =v(jj)+exp(alp(jj)+alp(jj+nr)+simshock(kk,(jj+1))*varvu(jj))*X(gm,jj)
			      else
				    v(jj) =-exp(alp(jj)+simshock(kk,(jj+1))*varvu(jj))*(X(gm,5)-X(gm,jj))	
				    v(jj) =v(jj)-exp(alp(jj)+alp(jj+nr)+simshock(kk,(jj+1))*varvu(jj))*X(gm,jj)
			      end if
	    end do
	    !(3) Calculate equilibrium in each participation case
	    do ii = 1,num
		  call sim_esearch(v,ptn(ii,:),thes,equthn(ii),payoff(ii,:),spend(ii,:))
           end do
	    !(4) Find equilibria by checking a deviation 
	    enum = 0
	    do ii = 1,num
	        jj = 1
	        dev = 0
	        equinx(ii) = 0
	        do while((jj<=nr).and.(dev.eq.0))
	            jjn = int(2.0d0**((nr-jj)*1.0d0))
	            util0 = payoff(ii,jj)			    
		     if (ptn(ii,jj)==0) then
		          util1 = payoff((ii+jjn),jj)
	            else
	                 util1 = payoff((ii-jjn),jj)
	            end if
	            if (util1>util0) then
	        	   dev = 1
		     end if
		     jj = jj+1
		 end do
		 if (dev.eq.0) then
                   enum = enum+1
                   equinx(enum) = ii
                   equsum(enum) = 0.0d0
                   do jj = 1,nr
                        equsum(enum) = equsum(enum)+payoff(ii,jj)
                   end do                    
		  end if
           end do
	    !(5) Find an equilibrium that is pareto optimal and check if the equilibrium is consistent
	    !      with the observed entry profile.		    
	    if (enum>0) then
                mdx = equinx(minloc(equsum(1:enum),1))
                do jj = 1,nr
                    lspd(jj) = lspd(jj) + spend(mdx,jj)/(ns*1.0d0)
      	         end do
		  ! lck is 0 if the selected equilibrium is equal to the observed equilibrium                
                lck = 0.0d0
                do jj = 1,nr
                    lck = lck+((ptn(mdx,jj)-rst_ptn(gm,jj))*1.0d0)**2.0d0
                end do
	    !(2-6) Calculate the likelihood
		  if (lck.eq.0.0d0) then
		      if (rst_psg(gm)==1) then
		          lcon = lcon + equthn(mdx)/(ns*1.0d0)
		      else
		          lcon = lcon + (1.0d0-equthn(mdx))/(ns*1.0d0)
		      end if
                end if
	     end if    
      end do

      if (lcon<toll) then
         print *, "Likelihood calculation error"
         lval = log(toll)
      else
         lval = log(lcon)
      end if

 end subroutine mvalue
