module esearch
use globvar
implicit none

contains
  
    subroutine sim_esearch(ves,pes,thes,conn,eprob,payoff,spd)
		implicit none

		real(8), intent(in) :: ves(nr)              !value
		integer, intent(in) :: pes(nr)              !ptn
		real(8), intent(in) :: thes                 !inital prob
		real(8), intent(in) :: conn(nr)             !connection
		real(8), intent(out) :: eprob               !psg prob
		real(8), intent(out) :: payoff(nr)          !payoff
		real(8), intent(out) :: spd(nr)             !spending

		real(8), parameter :: tol = 1.0E-8
		real(8), parameter :: tol2 = 1.0E-2
		integer :: i, ii, maxfn
		real(8) :: k, x1, x2, y1, y2, ecost
		real(8) :: tolc, isd, eqthn,adi
		integer :: fd, iter

		k = 0.0d0
		do i = 1,nr
			if ((pes(i).eq.1).and.(ves(i)>0.0d0)) then
				adi = bf*((bf*gam*ves(i))**(gam/(1.0d0-gam)))
				k = k + adi
			elseif ((pes(i).eq.1).and.(ves(i)<0.0d0)) then
				adi = ba*((ba*gam*(-ves(i)))**(gam/(1.0d0-gam)))
				k = k - adi
			end if
		end do

		fd = 0
		iter = 0
		tolc = tol

		do while ((fd.eq.0).and.(iter<20))
			if (k.ne.0) then
				x1 = -lam+lam*tolc
				x2 = lam-lam*tolc
				y1 = mainequ(x1,k,thes)
				y2 = mainequ(x2,k,thes)
				if (y1*y2<=0.0d0) then
					eqthn = zbrent(x1,x2,tol,k,thes)
					fd = 1
				elseif ((abs(y1)<tol).or.(abs(y2)<tol)) then
					if (abs(y1)<=abs(y2)) then
						eqthn = x1
						fd = 1
					else
						eqthn = x2
						fd = 1
					end if
				end if
			else
				eqthn = thes
				fd = 1
			end if
			tolc = tolc*tol2
			iter = iter+1
		end do
    
		if ((eqthn<=0.0d0).and.(eqthn>=-lam)) then
			isd = eqthn+lam
		elseif ((eqthn>0.0d0).and.(eqthn<=lam)) then
			isd = lam-eqthn
		else
		    print *, "equilibrium calculation problem"
		end if
		eprob = tricdf(eqthn)
		
		do ii = 1,nr
			ecost = ect+cect*conn(ii)
			if (ecost<0.0d0) then
			       ecost = 0.0d0
			end if

			if ((pes(ii)==1).and.(ves(ii)>0.0d0)) then
				spd(ii) = (isd*ves(ii)*bf*gam)**(1.0d0/(1.0d0-gam))+ ecost
				payoff(ii) = eprob*ves(ii)-spd(ii)
			elseif ((pes(ii)==1).and.(ves(ii)<0.0d0)) then
				spd(ii) = (isd*(-ves(ii))*ba*gam)**(1.0d0/(1.0d0-gam))+ecost
				payoff(ii) = eprob*ves(ii)-spd(ii)
			else
				spd(ii) = 0.0d0
				payoff(ii) = eprob*ves(ii)
			end if
		end do
	end subroutine sim_esearch


    real(8) function ZBRENT(X1,X2,TOL,k,thes)
      
      ! Using Brent's method, find the root of FUNC known to lie between X1 and X2. The root, returned as ZBRENT, will be
      ! refined until its accuracy is tol.
      
        integer, parameter :: ITMAX=100       ! Maximum number of iterations
        real(8), parameter :: EPS=3.E-7       ! Machine floating-point precision
        real(8) :: x1, x2, tol, k, thes

        real(8) :: a, b, fa, fb
        integer :: iter
        real(8) :: c, fc,d, e
        real(8) :: s, p, q, r
        real(8) :: tol1, xm
        
        A=X1
        B=X2
        FA=mainequ(A,k,thes)
        FB=mainequ(B,k,thes)
        if(FB*FA.GT.0.) then
            print*, 'Root must be bracketed for ZBRENT.'
            stop
        end if
        FC=FB
        do  ITER=1,ITMAX
            if (FB*FC.GT.0.) then
                C=A
                FC=FA
                D=B-A
                E=D
            end if
            if (ABS(FC).LT.ABS(FB)) then
                A=B
                B=C
                C=A
                FA=FB
                FB=FC
                FC=FA
            end if
            TOL1=2.*EPS*ABS(B)+0.5*TOL
            XM=.5*(C-B)
            IF (ABS(XM).LE.TOL1 .OR. FB.EQ.0.)THEN
                ZBRENT=B
                RETURN
            ENDIF
            IF (ABS(E).GE.TOL1 .AND. ABS(FA).GT.ABS(FB)) THEN
                S=FB/FA
                IF (A.EQ.C) THEN
                    P=2.*XM*S
                    Q=1.-S
                ELSE
                    Q=FA/FC
                    R=FB/FC
                    P=S*(2.*XM*Q*(Q-R)-(B-A)*(R-1.))
                    Q=(Q-1.)*(R-1.)*(S-1.)
                ENDIF
                IF(P.GT.0.) Q=-Q
                P=ABS(P)
                IF(2.*P .LT. MIN(3.*XM*Q-ABS(TOL1*Q),ABS(E*Q))) THEN
                    E=D
                    D=P/Q
                ELSE
                    D=XM
                    E=D
                ENDIF
            ELSE
                D=XM
                E=D
            ENDIF
            A=B
            FA=FB
            IF (ABS(D) .GT. TOL1) THEN
                B=B+D
            ELSE
                B=B+SIGN(TOL1,XM)
            ENDIF
            FB=mainequ(B,k,thes)
        end do
        print*, 'ZBRENT exceeding maximum iterations.'
        stop
!       ZBRENT=B
        RETURN
    end function ZBRENT
	
	real(8) function mainequ(y,k,thes)
		use globvar
		implicit none
		real(8), intent(in) :: y
		real(8), intent(in) :: k
		real(8), intent(in) :: thes

		mainequ = (lam-abs(y))**(gam/(1.0d0-gam))-(y-thes)/k
	end function mainequ

	real(8) function tricdf(y)
		use globvar
		implicit none
		real(8), intent(in) :: y
		
		if ((y<=0.0d0).and.(y>-lam)) then
			tricdf = ((y+lam)**2.0d0)/2.0d0
		elseif ((y>0.0d0).and.(y<lam)) then
			tricdf = 1.0d0-((y-lam)**2.0d0)/2.0d0
		elseif (y<=-lam) then
			tricdf = 0.0d0
		elseif (y>= lam) then
			tricdf = 1.0d0
		end if
	end function tricdf
	
	real(8) function tricdf_inv(y)
		use globvar
		implicit none
		real(8), intent(in) :: y
		
		if ((y<=0.5d0).and.(y>0.0d0)) then
			tricdf_inv = sqrt(y*2.0d0)-lam
		elseif ((y>0.5d0).and.(y<1.0d0)) then
			tricdf_inv = lam -sqrt((1.0d0-y)*2.0d0)
		elseif (y<=0.0d0) then
			tricdf_inv = -lam
		elseif (y>=1.0d0) then
			tricdf_inv = lam
		end if
	end function tricdf_inv	

    subroutine choldc(chn, carr)
    implicit none
    integer, intent(in) :: chn
    real(8), intent(inout) :: carr(chn,chn)
    integer :: ii, jj, kk
    real(8) :: summ, chp(chn)
    
    ! input: carr(chn,chn) is a symmetric and positive definite matrix.
    ! outout: carr(chn,chn) is a lower triangular matrix L, so that L*L' equals the original matrix.
    
    do ii = 1,chn
        do jj = ii, chn
            summ = carr(ii,jj)
            do kk= (ii-1),1,-1
                summ = summ - carr(ii,kk)*carr(jj,kk)
            end do
            if (ii.eq.jj) then
                if (summ.le.0.0d0) then
                    print *, "choldc failed"
                    return
                else
                    chp(ii) = sqrt(summ)
                end if
            else
                carr(jj,ii) = summ/chp(ii)
            endif
        enddo
    enddo

    do ii = 1,chn
        do jj = 1,chn
            if (ii<jj) then
                carr(ii,jj) = 0.0d0
            elseif (ii.eq.jj) then
                carr(ii,jj) = chp(ii)
            endif
        end do
    end do    
    
    return    
    end subroutine choldc

	SUBROUTINE findinv(matrix, inverse, n, errorflag)
	IMPLICIT NONE
	!Declarations
	INTEGER, INTENT(IN) :: n
	INTEGER, INTENT(OUT) :: errorflag  !Return error status. -1 for error, 0 for normal
	REAL(8), INTENT(IN), DIMENSION(n,n) :: matrix  !Input matrix
	REAL(8), INTENT(OUT), DIMENSION(n,n) :: inverse !Inverted matrix
	
	LOGICAL :: FLAG = .TRUE.
	INTEGER :: i, j, k, l
	REAL(8) :: m
	REAL(8), DIMENSION(n,2*n) :: augmatrix !augmented matrix
	
	!Augment input matrix with an identity matrix
	DO i = 1, n
		DO j = 1, 2*n
			IF (j <= n ) THEN
				augmatrix(i,j) = matrix(i,j)
			ELSE IF ((i+n) == j) THEN
				augmatrix(i,j) = 1
			Else
				augmatrix(i,j) = 0
			ENDIF
		END DO
	END DO
	
	!Reduce augmented matrix to upper traingular form
	DO k =1, n-1
		IF (augmatrix(k,k) == 0) THEN
			FLAG = .FALSE.
			DO i = k+1, n
				IF (augmatrix(i,k) /= 0) THEN
					DO j = 1,2*n
						augmatrix(k,j) = augmatrix(k,j)+augmatrix(i,j)
					END DO
					FLAG = .TRUE.
					EXIT
				ENDIF
				IF (FLAG .EQV. .FALSE.) THEN
					PRINT*, "Matrix is non - invertible"
					inverse = 0
					errorflag = -1
					return
				ENDIF
			END DO
		ENDIF
		DO j = k+1, n			
			m = augmatrix(j,k)/augmatrix(k,k)
			DO i = k, 2*n
				augmatrix(j,i) = augmatrix(j,i) - m*augmatrix(k,i)
			END DO
		END DO
	END DO
	
	!Test for invertibility
	DO i = 1, n
		IF (augmatrix(i,i) == 0) THEN
			PRINT*, "Matrix is non - invertible"
			inverse = 0
			errorflag = -1
			return
		ENDIF
	END DO
	
	!Make diagonal elements as 1
	DO i = 1 , n
		m = augmatrix(i,i)
		DO j = i , (2 * n)				
			   augmatrix(i,j) = (augmatrix(i,j) / m)
		END DO
	END DO
	
	!Reduced right side half of augmented matrix to identity matrix
	DO k = n-1, 1, -1
		DO i =1, k
		m = augmatrix(i,k+1)
			DO j = k, (2*n)
				augmatrix(i,j) = augmatrix(i,j) -augmatrix(k+1,j) * m
			END DO
		END DO
	END DO				
	
	!store answer
	DO i =1, n
		DO j = 1, n
			inverse(i,j) = augmatrix(i,j+n)
		END DO
	END DO
	errorflag = 0
    END SUBROUTINE findinv

end module esearch