subroutine moments(par_n, par, mom_n, mom)

use globvar
use esolver
use etoolbox
use csv_file
implicit none

integer, intent(in) :: par_n
real(8), intent(in) :: par(par_n)
integer, intent(in) :: mom_n
real(8), intent(out) :: mom(mom_n)

real(8), parameter :: toll = 1.0E-16

! outcome variables (aggregate)
! unconditional on ptn
real(8) :: noequ, numequ, conequ                        ! no pure equilibria, num of equilibria, consistent equilibria
real(8) :: upsg(3), uini(3), ueff(3)                    ! average passage rate - after/before lobbying (all/actual passage/actual failure) 
real(8) :: uspd(nr), uptn(nr), uiptn(nr), urptn(nr)     ! total spending, average ptn, irrelevant ptn, relevant ptn
real(8) :: uvalm(nr), uvalv(nr)                         ! mean and variance of value of policy
real(8) :: utrtn(nr), uartn(nr)                         ! avg return during the period, avg return per policy
real(8) :: uwelg(nr)                                    ! welfare gain of lobbying ban
real(8) :: ueffc(nr)                                    ! effect of lobbying of each player
real(8) :: uentry(nr,2)                                 ! entry cost
real(8) :: ucorr(nr,nr)                                 ! ptn correlation
real(8) :: ueffs, ueffa
! conditional on ptn
real(8) :: cpsg(3), cini(3), ceff(3)                    ! average passage rate - after/before lobbying (all/actual passage/actual failure) 
real(8) :: cspd(nr), cvalm(nr), cvalv(nr)               ! total spending, mean and variance of value of policy
real(8) :: ctrtn(nr), cartn(nr)                         ! avg return during the period, avg return per policy
real(8) :: cwelg(nr)                                    ! welfare gain of lobbying ban 
real(8) :: ceffc(nr)                                    ! effect of lobbying of each player
real(8) :: ceffs, ceffa
! conditional on categories
real(8) :: cat1_psg(ncat1), cat2_psg(ncat2)    ! average passage rate conditional on categories
real(8) :: cat1_ini(ncat1), cat2_ini(ncat2)    ! average initial probl conditional on categories
real(8) :: cat1_eff(ncat1), cat2_eff(ncat2)    ! average effect of lobbying conditional on categories
real(8) :: cat1_ptn(ncat1,nr), cat2_ptn(ncat2,nr)   ! average participation rate conditional on categories
real(8) :: cat2_effs(ncat2), cat2_effa(ncat2), cat2_can(ncat2)

! data summary
real(8) :: rst_aptn(nr), rst_iptn(nr), rst_rptn(nr)     ! data summary stats - ptn (average, irelevant, relevant)
real(8) :: rst_apsg, renum(nr), ienum(nr)
real(8) :: rst_c1psg(ncat1), rst_c2psg(ncat2)
real(8) :: rst_c1ptn(ncat1,nr), rst_c2ptn(ncat2,nr) 
real(8) :: rst_corr(nr,nr)

! list of policies by the effect
real(8) :: listeff(np,13)

! varibles for calculation
real(8) :: iz, thes, util1, util0, chk, iniprob
integer :: gg, ss, ii, jj, kk, cc, enum, dev, jjn, mdx, equinx(num), rn, in
real(8) :: v(nr), equprob(num), payoff(num,nr), spend(num,nr), equsum(num), diff(num,nr)
real(8) :: numlob(3), nck(3)

real(8) :: no_equ, con_equ, pck
real(8) :: u_psg, u_ini, u_eff, u_ptn(nr), u_rptn(nr), u_iptn(nr), u_spd(nr), u_trtn(nr)
real(8) :: u_artn(nr), u_welg(nr), u_valm(nr), u_valv(nr), u_entry(nr,2), u_effs, u_effa
real(8) :: c_psg, c_ini, c_eff, c_ref, c_spd(nr), c_trtn(nr), c_artn(nr), c_welg(nr)
real(8) :: c_valm(nr), c_valv(nr), u_effc(nr), c_effc(nr), c_effs, c_effa, c_can  
real(8) :: alt_thn1, alt_thn2, alt_prob, fspd, aspd, u_corr(nr,nr), stdev_ptn(nr), ef1, ef2
integer :: j1, j2



	! (0) Parameters
	
    call parscale_avar(par_n,par)
    
    pck = 0.0d0  
	if (bf.le.0.0d0) then
        pck = 1.0d0
        bf = toll
    endif
	if (ba.le.0.0d0) then
        pck = 1.0d0
        ba = toll
    endif
    if (gam.le.0.0d0) then
        pck = 1.0d0
        gam = toll
    endif
    if (gam.ge.1.0d0) then
        pck = 1.0d0
        gam = 1.0d0-toll
    endif
    if (varin.le.0.0d0) then
        pck = 1.0d0
        varin = 0.0d0
    endif
	do jj = 1,nr
	    if (varvu(jj).le.0.0d0) then
            pck = 1.0d0
	        varvu(jj) = 0.0d0
	    end if
	end do
	
	! (1) initialize variables

    do ii = 1,mom_n
        mom(ii) = 0.0d0
    end do

    noequ = 0.0d0
    numequ = 0.0d0
    conequ = 0.0d0
    do jj = 1,3
        upsg(jj) = 0.0d0
        uini(jj) = 0.0d0
        ueff(jj) = 0.0d0
        cpsg(jj) = 0.0d0
        cini(jj) = 0.0d0
        ceff(jj) = 0.0d0
    end do
    do ii = 1,ncat1
        cat1_psg(ii) = 0.0d0
        cat1_ini(ii) = 0.0d0
        cat1_eff(ii) = 0.0d0        
        do jj = 1,nr
            cat1_ptn(ii,jj) = 0.0d0
        end do     
    end do
    do ii = 1,ncat2
        cat2_psg(ii) = 0.0d0
        cat2_ini(ii) = 0.0d0
        cat2_eff(ii) = 0.0d0
        cat2_effs(ii) = 0.0d0
        cat2_effa(ii) = 0.0d0
        cat2_can(ii) = 0.0d0
        do jj = 1,nr
            cat2_ptn(ii,jj) = 0.0d0
        end do   
    end do  

    do jj = 1,nr
        uspd(jj) = 0.0d0
        uptn(jj) = 0.0d0
        uiptn(jj) = 0.0d0
        urptn(jj) = 0.0d0
        uvalm(jj) = 0.0d0
        uvalv(jj) = 0.0d0
        utrtn(jj) = 0.0d0
        uartn(jj) = 0.0d0
        uwelg(jj) = 0.0d0
        ueffc(jj) = 0.0d0
        uentry(jj,1) = 0.0d0
        uentry(jj,2) = 0.0d0
        cspd(jj) = 0.0d0
        cvalm(jj) = 0.0d0
        cvalv(jj) = 0.0d0
        ctrtn(jj) = 0.0d0
        cartn(jj) = 0.0d0
        cwelg(jj) = 0.0d0
        ceffc(jj) = 0.0d0
    end do 
    
    do j1= 1,nr
        do j2 = 1,nr
            ucorr(j1,j2) = 0.0d0
        end do
    end do
    
    ueffs = 0.0d0
    ueffa = 0.0d0
    ceffs = 0.0d0
    ceffa = 0.0d0    
    
    numlob(1) = 0.0d0
    numlob(2) = 0.0d0
    numlob(3) = 0.0d0
    do gg = 1,np
        nck(1) = 0.0d0
        nck(2) = 0.0d0
        nck(3) = 0.0d0
        do jj = 1,nr
            if (rst_ptn(gg,jj) == 1) then
                nck(1) = 1.0d0
                if (W(gg,jj) == 1) then
                    nck(2) = 1.0d0
                else
                    nck(3) = 1.0d0
                end if
             end if
        end do
        numlob(1) = numlob(1) + nck(1)
        numlob(2) = numlob(2) + nck(2)
        numlob(3) = numlob(3) + nck(3)
    end do
    
!-------------------------------------------------------------
! (2) Equilibrium for each case
!-------------------------------------------------------------

    do gg = 1,np 
        u_psg = 0.0d0
        u_ini = 0.0d0
        u_eff = 0.0d0
        c_psg = 0.0d0
        c_ini = 0.0d0
        c_eff = 0.0d0
        c_ref = 0.0d0     
        do jj = 1,nr
            u_ptn(jj) = 0.0d0
            u_iptn(jj) = 0.0d0
            u_rptn(jj) = 0.0d0
            u_spd(jj) = 0.0d0
            u_trtn(jj) = 0.0d0
            u_artn(jj) = 0.0d0
            u_welg(jj) = 0.0d0
            u_valm(jj) = 0.0d0
            u_valv(jj) = 0.0d0
            u_effc(jj) = 0.0d0
            c_spd(jj) = 0.0d0
            c_trtn(jj) = 0.0d0
            c_artn(jj) = 0.0d0
            c_welg(jj) = 0.0d0
            c_valm(jj) = 0.0d0
            c_valv(jj) = 0.0d0  
            c_effc(jj) = 0.0d0          
        end do
        do jj = 1,nr
            u_entry(jj,1) = ect
            u_entry(jj,2) = ect**2.0d0
        end do
        do j1= 1,nr
            do j2 = 1,nr
                u_corr(j1,j2) = 0.0d0
            end do
        end do
        
        u_effs = 0.0d0
        u_effa = 0.0d0
        c_effs = 0.0d0
        c_effa = 0.0d0
        c_can = 0.0d0
        
        no_equ = 0.0d0
        con_equ = 0.0d0        
	 do ss = 1,ns
	       kk = ns*(gg-1) + ss
	       !(2-1) Generate theta's
	       iz = 0.0
	       do ii = 1,6
	             iz = iz + Z(gg,ii)*del(ii)
	       end do
	       thes = iz+varin*simshock(kk,1)
	       iniprob = al_norm(thes,.False.)

	      !(2-2) Generate values
	       do jj = 1,nr
			if (W(gg,jj).eq.1) then
				v(jj) =exp(alp(jj)+alp(jj+nr)*X(gg,jj)+simshock(kk,(jj+1))*varvu(jj))
			else
				v(jj) =-exp(alp(jj)+alp(jj+nr)*X(gg,jj)+simshock(kk,(jj+1))*varvu(jj))
			end if
	       end do

	      !(2-3) Calculate equilibrium in each participation case

		do ii = 1,num
			call sim_esearch(v,ptn(ii,:),thes,equprob(ii),payoff(ii,:),spend(ii,:))
		end do

		!(2-4) Find equilibria by checking a deviation
		enum = 0
		do ii = 1,num
		     equinx(ii) = 0
                   jj = 1
                   dev = 0
		     do while((jj<=nr).and.(dev.eq.0))
		          jjn = int(2.0d0**((nr-jj)*1.0d0))
			   util0 = payoff(ii,jj)			    
		          if (ptn(ii,jj)==0) then
		               util1 = payoff((ii+jjn),jj)
		          else
		               util1 = payoff((ii-jjn),jj)
		          end if
		          diff(ii,jj) = util0-util1
			   if (util1>util0) then
			        dev = 1
		          end if
			   jj = jj+1
		     end do
		     if (dev.eq.0) then
                         enum = enum+1
                         equinx(enum) = ii
                         equsum(enum) = 0.0d0
                         do jj = 1,nr
                              equsum(enum) = equsum(enum)+payoff(ii,jj)
                         end do                    
		     end if
              end do

		!(2-5) Find an equilibrium that is pareto optimal and check if the equilibrium is consistent
		!    with the observed entry profile.
		    
		if (enum.eq.0) then
                    noequ = noequ+1.0d0/(ns*np*1.0d0)
                    no_equ = no_equ + 1.0d0			
		else
	             numequ = numequ+enum*1.0d0/(ns*np*1.0d0)                      
                    mdx = equinx(minloc(equsum(1:enum),1))          ! mdx: index number of the selected equilibrium
                    ! chk is 0 if the selected equilibrium is equal to the observed equilibrium                
                    chk = 0.0d0
                    do jj = 1,nr
                          chk = chk+((ptn(mdx,jj)-rst_ptn(gg,jj))*1.0d0)**2.0d0
                    end do

                    ! unconditional aggregate variables
                    u_psg = u_psg + equprob(mdx)
                    u_ini = u_ini + iniprob
                    u_eff = u_eff + abs(equprob(mdx)-iniprob)
                    fspd = 0.0d0
                    aspd = 0.0d0             
                    do jj = 1,nr
			   u_valm(jj) = u_valm(jj) + abs(v(jj))
			   u_valv(jj) = u_valv(jj) + v(jj)**2.0d0                
                        u_ptn(jj) = u_ptn(jj)+ ptn(mdx,jj)*1.0d0
                        if (X(gg,jj) == 1.0d0) then
                             u_rptn(jj) = u_rptn(jj)+ ptn(mdx,jj)*1.0d0
                        else
                             u_iptn(jj) = u_iptn(jj)+ ptn(mdx,jj)*1.0d0
                        end if
                        u_spd(jj) = u_spd(jj)+ spend(mdx,jj) 
                        u_trtn(jj) = u_trtn(jj)+ diff(mdx,jj)*(ptn(mdx,jj)*1.0d0)
                        if (ptn(mdx,jj)==1) then
                              u_artn(jj) = u_artn(jj)+ diff(mdx,jj)/spend(mdx,jj)
                              if (W(gg,jj) == 1) then
                                   fspd = fspd + bf*((spend(mdx,jj)-ect)**gam)
                              else
                                   aspd = aspd + ba*((spend(mdx,jj)-ect)**gam)
                              end if
                         end if
                     end do
                     u_effs = u_effs + abs(iniprob-(iniprob+fspd)/(1.0d0+fspd))
                     u_effa = u_effa + abs(iniprob-iniprob/(1.0d0+aspd))
                     do jj = 1,nr
                          if (ptn(mdx,jj)== 1) then
                               if (W(gg,jj) == 1) then
                                    alt_thn1 = iniprob + fspd - bf*((spend(mdx,jj)-ect)**gam)
                                    alt_thn2 = 1.0d0 + fspd + aspd - bf*((spend(mdx,jj)-ect)**gam)
                                    alt_prob = alt_thn1/alt_thn2
                                    u_effc(jj) = u_effc(jj) + abs(equprob(mdx)-alt_prob)
                               else
                                    alt_thn1 = iniprob + fspd
                                    alt_thn2 = 1.0d0 + fspd + aspd - ba*((spend(mdx,jj)-ect)**gam)
                                    alt_prob = alt_thn1/alt_thn2
                                    u_effc(jj) = u_effc(jj) + abs(equprob(mdx)-alt_prob)
                               end if
                           end if                                    
                      end do
  
                      do j1= 1,nr
                           do j2 = 1,nr
                                u_corr(j1,j2) = u_corr(j1,j2) + ptn(mdx,j1)*ptn(mdx,j2)
                           end do
                      end do

                      ! conditional aggregate variables                
			 if (chk==0.0d0) then
			     conequ = conequ+1.0d0/(ns*np*1.0d0)		    
			     con_equ = con_equ+1.0d0
			     c_psg = c_psg + equprob(mdx)
			     c_ini = c_ini + iniprob
			     c_eff = c_eff + abs(equprob(mdx)-iniprob)				 					
			     c_ref = c_ref + equprob(mdx)-iniprob
			     fspd = 0.0d0
			     aspd = 0.0d0
			     do jj = 1,nr
			          c_valm(jj) = c_valm(jj) + abs(v(jj))
			          c_valv(jj) = c_valv(jj) + v(jj)**2.0d0  					
                               c_spd(jj) = c_spd(jj)+ spend(mdx,jj)
                               c_trtn(jj) = c_trtn(jj)+ diff(mdx,jj)*(ptn(mdx,jj)*1.0d0)
                               if (ptn(mdx,jj)==1) then
                                     c_artn(jj) = c_artn(jj) + diff(mdx,jj)/spend(mdx,jj)
                                     if (W(gg,jj) == 1) then
                                           fspd = fspd + bf*((spend(mdx,jj)-ect)**gam)
                                      else
                                           aspd = aspd + ba*((spend(mdx,jj)-ect)**gam)
                                      end if
                                end if
                                c_welg(jj) = c_welg(jj)+ iniprob*v(jj)-payoff(mdx,jj)                  
			     end do
                          ef1 = abs(iniprob-(iniprob+fspd)/(1.0d0+fspd))
                          ef2 = abs(iniprob-iniprob/(1.0d0+aspd))
                          c_effs = c_effs + ef1
                          c_effa = c_effa + ef2
                          if (ef1+ef2>0.0d0) then 
                              c_can = c_can + min(ef1,ef2)/(ef1+ef2)
                          end if
                          do jj = 1,nr
                                if (ptn(mdx,jj)== 1) then
                                     if (W(gg,jj) == 1) then
                                           alt_thn1 = iniprob + fspd - bf*((spend(mdx,jj)-ect)**gam)
                                           alt_thn2 = 1.0d0 + fspd + aspd - bf*((spend(mdx,jj)-ect)**gam)
                                           alt_prob = alt_thn1/alt_thn2
                                           c_effc(jj) = c_effc(jj) + abs(equprob(mdx)-alt_prob)
                                      else
                                           alt_thn1 = iniprob + fspd
                                           alt_thn2 = 1.0d0 + fspd + aspd - ba*((spend(mdx,jj)-ect)**gam)
                                           alt_prob = alt_thn1/alt_thn2
                                           c_effc(jj) = c_effc(jj) + abs(equprob(mdx)-alt_prob)
                                      end if                       
                                  end if               
                           end do
		     end if
		end if													 
	end do
		
	!(3) sum over all observations
	if (con_equ>0) then
	    listeff(gg,1) = gg*1.0d0
	    listeff(gg,2) = c_psg/con_equ
	    listeff(gg,3) = c_ini/con_equ
	    listeff(gg,4) = c_eff/con_equ
	    listeff(gg,5) = c_ref/con_equ
	    cpsg(1) = cpsg(1) + c_psg/(con_equ*(np*1.0d0))
	    cini(1) = cini(1) + c_ini/(con_equ*(np*1.0d0))
	    ceff(1) = ceff(1) + c_eff/(con_equ*(numlob(1)*1.0d0))		    
	    cat1_psg(Ct(gg,1)) = cat1_psg(Ct(gg,1))+ c_psg/(con_equ*(Cn(gg,1)*1.0d0))
	    cat2_psg(Ct(gg,2)) = cat2_psg(Ct(gg,2))+ c_psg/(con_equ*(Cn(gg,2)*1.0d0))		    
	    cat1_ini(Ct(gg,1)) = cat1_ini(Ct(gg,1))+ c_ini/(con_equ*(Cn(gg,1)*1.0d0))
	    cat2_ini(Ct(gg,2)) = cat2_ini(Ct(gg,2))+ c_ini/(con_equ*(Cn(gg,2)*1.0d0))
	    cat1_eff(Ct(gg,1)) = cat1_eff(Ct(gg,1))+ c_eff/(con_equ*(Cn(gg,1)*1.0d0))
	    cat2_eff(Ct(gg,2)) = cat2_eff(Ct(gg,2))+ c_eff/(con_equ*(Cn(gg,2)*1.0d0))	    
	    cat2_effs(Ct(gg,2)) = cat2_effs(Ct(gg,2))+ c_effs/(con_equ*(Cn(gg,2)*1.0d0))	
	    cat2_effa(Ct(gg,2)) = cat2_effa(Ct(gg,2))+ c_effa/(con_equ*(Cn(gg,2)*1.0d0))
           cat2_can(Ct(gg,2)) = cat2_can(Ct(gg,2))+ c_can/(con_equ*(Cn(gg,2)*1.0d0))
	    if (rst_psg(gg) == 1) then
	        cpsg(2) = cpsg(2) + c_psg/con_equ
	        cini(2) = cini(2) + c_ini/con_equ
	        ceff(2) = ceff(2) + c_eff/con_equ
	    else
	        cpsg(3) = cpsg(3) + c_psg/con_equ
	        cini(3) = cini(3) + c_ini/con_equ
	        ceff(3) = ceff(3) + c_eff/con_equ
	    end if
	    do jj = 1,nr
	        listeff(gg,5+jj) = c_valm(jj)/con_equ
	        listeff(gg,9+jj) = c_spd(jj)/con_equ
	        cvalm(jj) = cvalm(jj) + c_valm(jj)/con_equ
	        cvalv(jj) = cvalv(jj) + c_valv(jj)/con_equ		         
	        cspd(jj) = cspd(jj) + c_spd(jj)/con_equ
	        ctrtn(jj) = ctrtn(jj) + c_trtn(jj)/con_equ
	        cartn(jj) = cartn(jj) + c_artn(jj)/con_equ
	        cwelg(jj) = cwelg(jj) + c_welg(jj)/con_equ
	        ceffc(jj) = ceffc(jj) + c_effc(jj)/con_equ       	        
	    end do
	    ceffs = ceffs + c_effs/(con_equ*numlob(2))
	    ceffa = ceffa + c_effa/(con_equ*numlob(3))		        
	end if
			
	if (no_equ<(ns*1.0d0)) then
	    upsg(1) = upsg(1) + u_psg/((ns-no_equ)*(np*1.0d0))
	    uini(1) = uini(1) + u_ini/((ns-no_equ)*(np*1.0d0))
	    ueff(1) = ueff(1) + u_eff/((ns-no_equ)*(np*1.0d0))
	    if (rst_psg(gg) == 1) then
	        upsg(2) = upsg(2) + u_psg/(ns-no_equ)
	        uini(2) = uini(2) + u_ini/(ns-no_equ)
	        ueff(2) = ueff(2) + u_eff/(ns-no_equ)
	    else
	        upsg(3) = upsg(3) + u_psg/(ns-no_equ)
	        uini(3) = uini(3) + u_ini/(ns-no_equ)
	        ueff(3) = ueff(3) + u_eff/(ns-no_equ)
	    end if
	    do jj = 1,nr
	        uvalm(jj) = uvalm(jj) + u_valm(jj)/((ns-no_equ)*1.0d0)
	        uvalv(jj) = uvalv(jj) + u_valv(jj)/((ns-no_equ)*1.0d0)		    
	        uptn(jj) = uptn(jj)+u_ptn(jj)/((ns-no_equ)*(np*1.0d0))
	        uiptn(jj)= uiptn(jj)+u_iptn(jj)/((ns-no_equ)*1.0d0)
	        urptn(jj)= urptn(jj)+u_rptn(jj)/((ns-no_equ)*1.0d0)
	        uspd(jj) = uspd(jj) + u_spd(jj)/((ns-no_equ)*1.0d0)
	        utrtn(jj) = utrtn(jj) + u_trtn(jj)/((ns-no_equ)*1.0d0)
	        uartn(jj) = uartn(jj) + u_artn(jj)/((ns-no_equ)*1.0d0)
	        uwelg(jj) = uwelg(jj) + u_welg(jj)/((ns-no_equ)*1.0d0)
	        ueffc(jj) = ueffc(jj) + u_effc(jj)/(u_ptn(jj)*(np*1.0d0)) 
	        cat1_ptn(Ct(gg,1),jj) = cat1_ptn(Ct(gg,1),jj)+ u_ptn(jj)/((ns-no_equ)*(Cn(gg,1)*1.0d0))
	        cat2_ptn(Ct(gg,2),jj) = cat2_ptn(Ct(gg,2),jj)+ u_ptn(jj)/((ns-no_equ)*(Cn(gg,2)*1.0d0))		        
	    end do
	    do j1 = 1,nr
	        do j2= 1,nr
	            ucorr(j1,j2) = ucorr(j1,j2) + u_corr(j1,j2)/(ns-no_equ)
	        end do
	    end do
	    ueffs = ueffs + u_effs/((ns-no_equ)*1.0d0*np)
	    ueffa = ueffa + u_effa/((ns-no_equ)*1.0d0*np)
	end if	
	
	do jj = 1,nr
	    uentry(jj,1) = uentry(jj,1) + u_entry(jj,1)
	    uentry(jj,2) = uentry(jj,2) + u_entry(jj,2)
	end do	
   end do
	
    do jj = 1,nr
	    uvalm(jj) = uvalm(jj)/(np*1.0d0)
	    uvalv(jj) = sqrt((uvalv(jj)/((np-1)*1.0d0)) - (uvalm(jj)**2.0d0)*(np*1.0d0)/((np-1)*1.0d0))
	    cvalm(jj) = cvalm(jj)/(np*1.0d0)
	    cvalv(jj) = sqrt((cvalv(jj)/((np-1)*1.0d0)) - (cvalm(jj)**2.0d0)*(np*1.0d0)/((np-1)*1.0d0))	
	    uvalm(jj) = uvalm(jj)/unitm
	    uvalv(jj) = uvalv(jj)/unitm
	    cvalm(jj) = cvalm(jj)/unitm
	    cvalv(jj) = cvalv(jj)/unitm
	    if (uspd(jj)>0.0d0) then    
	        utrtn(jj) = utrtn(jj)/uspd(jj)
	    else
	        utrtn(jj) = 0.0d0
	    endif
	    if (cspd(jj)>0.0d0) then
	        ctrtn(jj) = ctrtn(jj)/cspd(jj)
	    else
	        ctrtn(jj) = 0.0d0
	    endif
	    if (uptn(jj)>0.0d0) then
	        uartn(jj) = cartn(jj)/((np*1.0d0)*uptn(jj))
	    else
	        uartn(jj) = 0.0d0
	    endif
    end do
	
    do j1 = 1,nr
	    do j2 = 1,nr
	        ucorr(j1,j2) = ucorr(j1,j2)/((np-1)*1.0d0)-uptn(j1)*uptn(j2)*(np*1.0d0)/((np-1)*1.0d0)
	    end do
    end do
    do jj = 1,nr
	    stdev_ptn(jj) = sqrt(ucorr(jj,jj))
    end do
	
	do j1= 1,nr
	    do j2 = 1,nr
	        ucorr(j1,j2) = ucorr(j1,j2)/(stdev_ptn(j1)*stdev_ptn(j2))
	    end do
	end do
	
	! (4) display the results
	
	do jj = 1,nr
	    rst_aptn(jj) = 0.0d0
	    rst_rptn(jj) = 0.0d0
	    rst_iptn(jj) = 0.0d0
	end do
	rst_apsg = 0.0d0
	do cc = 1,ncat1
	    rst_c1psg(cc) = 0.0d0
	    do jj = 1,nr
	        rst_c1ptn(cc,jj) = 0.0d0
	    end do   
	end do
	do cc = 1,ncat2
	    rst_c2psg(cc) = 0.0d0
	    do jj = 1,nr
	        rst_c2ptn(cc,jj) = 0.0d0
	    end do   
	end do	
	do jj = 1,nr
	    renum(jj) = 0.0d0
	    ienum(jj) = 0.0d0
	end do
	do j1 = 1,nr
	    do j2 = 1,nr
	        rst_corr(j1,j2) = 0.0d0
	    end do
	end do		
	do gg = 1,np
	    rst_apsg = rst_apsg + (1.0d0*rst_psg(gg))/(1.0d0*np)
           rst_c1psg(Ct(gg,1)) = rst_c1psg(Ct(gg,1)) + (1.0d0*rst_psg(gg))/(1.0d0*Cn(gg,1))
           rst_c2psg(Ct(gg,2)) = rst_c2psg(Ct(gg,2)) + (1.0d0*rst_psg(gg))/(1.0d0*Cn(gg,2))       
           do jj = 1,nr
                if (X(gg,jj)==1) then
                     renum(jj) = renum(jj) + 1.0d0
                else
                     ienum(jj) = ienum(jj) + 1.0d0
                end if
           end do  
           do jj = 1,nr
                rst_aptn(jj) = rst_aptn(jj) + (1.0d0*rst_ptn(gg,jj))/(1.0d0*np)
                if (X(gg,jj)==1) then
                    rst_rptn(jj) = rst_rptn(jj) + (1.0d0*rst_ptn(gg,jj))
                else
                    rst_iptn(jj) = rst_iptn(jj) + (1.0d0*rst_ptn(gg,jj))
                end if            
                rst_c1ptn(Ct(gg,1),jj) = rst_c1ptn(Ct(gg,1),jj)+ (1.0d0*rst_ptn(gg,jj))/(1.0d0*Cn(gg,1))
                rst_c2ptn(Ct(gg,2),jj) = rst_c2ptn(Ct(gg,2),jj)+ (1.0d0*rst_ptn(gg,jj))/(1.0d0*Cn(gg,2))
           end do
           do j1 = 1,nr
                do j2 = 1,nr
                    rst_corr(j1,j2) = rst_corr(j1,j2) + (1.0d0*rst_ptn(gg,j1))*(1.0d0*rst_ptn(gg,j2))
                end do
           end do
       end do	
		
	upsg(2) = upsg(2)/(rst_apsg*(1.0d0*np))
	uini(2) = uini(2)/(rst_apsg*(1.0d0*np))
	ueff(2) = ueff(2)/(rst_apsg*(1.0d0*np))	
	cpsg(2) = cpsg(2)/(rst_apsg*(1.0d0*np))
	cini(2) = cini(2)/(rst_apsg*(1.0d0*np))
	ceff(2) = ceff(2)/(rst_apsg*(1.0d0*np))	
	upsg(3) = upsg(3)/((1.0d0-rst_apsg)*(1.0d0*np))
	uini(3) = uini(3)/((1.0d0-rst_apsg)*(1.0d0*np))
	ueff(3) = ueff(3)/((1.0d0-rst_apsg)*(1.0d0*np))	
	cpsg(3) = cpsg(3)/((1.0d0-rst_apsg)*(1.0d0*np))
	cini(3) = cini(3)/((1.0d0-rst_apsg)*(1.0d0*np))
	ceff(3) = ceff(3)/((1.0d0-rst_apsg)*(1.0d0*np))	
	do jj = 1,nr
	    uiptn(jj) = uiptn(jj)/ienum(jj)
	    urptn(jj) = urptn(jj)/renum(jj)
	    rst_iptn(jj) = rst_iptn(jj)/ienum(jj)
	    rst_rptn(jj) = rst_rptn(jj)/renum(jj)
	    cartn(jj) = cartn(jj)/(rst_aptn(jj)*(1.0d0*np))
	end do

       do jj = 1,nr
	    ceffc(jj) = ceffc(jj)/(rst_aptn(jj)*(numlob(1)*1.0d0))
	    uentry(jj,1) = uentry(jj,1)/(np*1.0d0)
	    uentry(jj,2) = sqrt(uentry(jj,2)/((np-1)*1.0d0)-(uentry(jj,1)**2.0d0)*((np*1.0d0)/((np-1)*1.0d0)))	    	    
	end do	
	
	do j1= 1,nr
	    do j2= 1,nr
	        rst_corr(j1,j2) = rst_corr(j1,j2)/((np-1)*1.0d0) - rst_aptn(j1)*rst_aptn(j2)*((np*1.0d0)/((np-1)*1.0d0))
	    end do
	end do
	do jj = 1,nr
	    stdev_ptn(jj) = sqrt(rst_corr(jj,jj))
	end do	
	do j1= 1,nr
	    do j2= 1,nr
	        rst_corr(j1,j2) = rst_corr(j1,j2)/(stdev_ptn(j1)*stdev_ptn(j2))
	    end do
	end do
	
	! (5) matrix
	
	mom(1) = noequ
	mom(2) = numequ
	mom(3) = conequ
	do jj = 1,nr
	    mom(3+jj) = uptn(jj)
	    mom(7+jj) = uspd(jj)
	    mom(11+jj) = uvalm(jj)
	    mom(15+jj) = uvalv(jj)
	    mom(19+jj) = utrtn(jj)
	    mom(23+jj) = uartn(jj)
	end do 
	do ii = 1,3
	    mom(27+ii) = cpsg(ii)
	    mom(30+ii) = ceff(ii)
	end do
	mom(34) = ceffs
	mom(35) = ceffa
	do ii = 1,ncat1
	    mom(35+ii) = cat1_psg(ii)
	    mom(38+ii) = cat1_eff(ii)
	end do
	do ii = 1,ncat2
	    mom(41+ii) = cat2_psg(ii)
	    mom(45+ii) = cat2_eff(ii)
	end do
       mom(50) = cat2_effs(1)
       mom(51) = cat2_effa(1)
       mom(52) = cat2_can(1)
	mom(53) = pck
       do ii= 1,3
          mom(53+ii) = ueff(ii)
       end do
                   
end subroutine moments