subroutine asymvar(par_n, par)

! This subroutine calculates the asymptotic variance-covariance of the estimated parameters

use globvar
use csv_file
use etoolbox

implicit none

integer, intent(in) :: par_n
real(8), intent(in) :: par(par_n)

real(8), parameter :: bump = 0.5d-4
real(8), dimension(par_n) :: par0, par1, par2
real(8), dimension(par_n) :: score, stderr
real(8), dimension(par_n, nr) :: mscores
real(8), dimension(par_n, par_n) :: var_score, ehessian, ihessian
real(8), dimension(par_n, par_n) :: var, varmat, stdmat
real(8) :: step1, step2, lv1, lv2, sp1(nr), sp2(nr)
integer :: gg, jj, vi, vj, inv_err

    do vi = 1,par_n
	 do jj= 1,nr
	    mscores(vi,jj) = 0.0d0
	 end do
	    do vj = 1,par_n 
		var_score(vi,vj) = 0.0d0
	    end do
    end do
	
    ! have the scaling and the reparametrization fixed
    call parscale(par_n,par)
    call parscale_inv(par0)
	
    do gg = 1,np
        do vi = 1,par_n
            par1 = par0
            par2 = par0
            step1 = abs(par0(vi))*bump
            par1(vi) = par1(vi)+step1
            par2(vi) = par2(vi)-step1
            call mvalue(par_n,par1,gg,lv1,sp1)
            call mvalue(par_n,par2,gg,lv2,sp2)
            score(vi) = (lv1-lv2)/(2.0d0*step1)
            do jj = 1,nr
                 mscores(vi,jj) = mscores(vi,jj)+(sp1(jj)-sp2(jj))/(2.0d0*step1*np)
            end do
        end do
	    do vi = 1, par_n
	        do vj =1, par_n
	            var_score(vi,vj) = var_score(vi,vj) + score(vi)*score(vj)/(np*1.0d0)
	        end do
	    end do
    end do

    do vi = 1, par_n
         do vj =1, par_n
	       ehessian(vi,vj) = -var_score(vi,vj)
		do jj = 1,nr
		    ehessian(vi,vj) = ehessian(vi,vj) - lagr(jj)*mscores(vi,jj)*mscores(vj,jj)
		end do
	  end do
    end do
	
    print *, "hessian and var(score) are calculated"
	
    call findinv(ehessian,ihessian,par_n,inv_err)
	
    if (inv_err == 0) then
	 print *, "inverse calculated successfully"
    else
	 print *, "inverse calculated with an error"
    end if

    ! Variance matrix
    
    do vi = 1,par_n
        do vj = 1,par_n
            var(vi,vj) = 0.0d0
            do jj = 1,par_n
                var(vi,vj) = var(vi,vj)+ihessian(vi,jj)*var_score(jj,vj)
            end do
        end do
    end do
	
    do vi = 1,par_n
        do vj = 1,par_n
            varmat(vi,vj) = 0.0d0
            do jj = 1,par_n
                varmat(vi,vj) = varmat(vi,vj)+var(vi,jj)*ihessian(jj,vj)
            end do
        end do
    end do

    do vi = 1,par_n
	  stderr(vi) = varmat(vi,vi)**(0.5d0)
    end do

    call parscale(par_n,par)
    
    write(1000,*) "parameter estimates"
    write(1000,*) "bump size:", bump
    write(1000,*) "simulation size:", ns
    do vi = 1,npar
        write(1000,*) par0(vi), stderr(vi)
    end do

    open(4000, file = "var_matrices.csv")    
    call csv_write_dble_2d(4000,ehessian)
    call csv_write_dble_2d(4000,ihessian)
    call csv_write_dble_2d(4000,var_score)
    close(4000)
    
end subroutine asymvar