module esolver
use globvar
use etoolbox
implicit none

contains
	subroutine sim_esearch(ves,pes,tes,eprob,payoff,spd)
		implicit none

	! This subroutine calculate equilibrium outcomes for each value and ptn matrix

		real(8), intent(in) :: ves(nr)              !value
		integer, intent(in) :: pes(nr)              !ptn
		real(8), intent(in) :: tes                  !inital prob
		real(8), intent(out) :: eprob               !psg prob
		real(8), intent(out) :: payoff(nr)          !payoff
		real(8), intent(out) :: spd(nr)             !spending

		real(8), parameter :: tol = 1.0E-12
		integer, parameter :: niter = 50
        integer :: i, ii, iter, erry, erryf, errya, errym
        real(8) :: x1, x2, y1, y2, ess, thes, cfa, cyc
        real(8) :: cf, ca, mf, ma, eqyf, eqya, eqys(2), entrycost		

        !---------------------------------------------
        ! (0) Initial enactment probability
        !---------------------------------------------
        
        thes = al_norm(tes,.False.)

        !----------------------------------------------------------
        ! (1) Solve nonlinear equations for Y_f* and Y_a* (Newton)
        !----------------------------------------------------------
		cf = 0.0d0
		ca = 0.0d0
		mf = 0.0d0
		ma = 0.0d0
		do i = 1,nr
			if ((pes(i).eq.1).and.(ves(i)>0.0d0)) then
				cf = cf+ bf*((bf*gam*ves(i))**(gam/(1.0d0-gam)))
				mf = mf+ bf*(ves(i)**gam)
			elseif ((pes(i).eq.1).and.(ves(i)<0.0d0)) then
				ca = ca+ ba*((ba*gam*(-ves(i)))**(gam/(1.0d0-gam)))
				ma = ma +ba*((-ves(i))**gam)
			end if
		end do
		
		mf = max(cf,mf)
		ma = max(ca,ma)
				
		erry = 0
		
		if ((cf.eq.0.0d0).and.(ca.eq.0.0d0)) then
		    eqyf = 0.0d0
		    eqya = 0.0d0
		elseif (cf.eq.0.0d0) then
		    eqyf = 0.0d0
		    call newton_aa(eqya,erry,tol,ca,thes)
		    if (erry.eq.1) then
		        eqya = brent_aa(tol,ma,tol,ca,thes)
		    endif
		elseif (ca.eq.0.0d0) then
		    eqya = 0.0d0
		    call newton_ff(eqyf,erry,tol,cf,thes)
		    if (erry.eq.1) then
		        eqyf = brent_ff(tol,mf,tol,cf,thes)
		    endif		      
		else
            call newton_fa(eqys,erry,tol,cf,ca,thes)
            eqyf = eqys(1)
            eqya = eqys(2)	
            
        !----------------------------------------------------------
        ! (2) Solve nonlinear equations for Y_f* and Y_a* (Brent)
        !----------------------------------------------------------            
            			
            if (erry.eq.1) then
                
                ! (2-1) min(S*)
                cfa = max((thes**(gam/(1.0d0-gam)))*ca,((1.0d0-thes)**(gam/(1.0d0-gam)))*cf)
                call newton_s0(x1,errym,tol,cfa)
                if (errym.eq.1) then
                    x1 = brent_s0(tol,cfa)
                end if              
                
                ! (2-2) max(S*)
                x2 = mf+ma
                ! (2-3) solve S* using brent
                y1 = esequ(x1,cf,ca,thes)
                y2 = esequ(x2,cf,ca,thes) 
                if (y1*y2.le.0.0d0) then               
                    ess = brent_es(x1,x2,tol,cf,ca,thes)
                elseif (y1>-tol) then
                    ess = x1
                elseif (y2<tol) then
                    ess = x2
                else
                    print *, "Wrong bracket for Brent_FA."
                endif
                ! (2-4) solve Y_f* given S*
                cyc = ess/ca-(thes/((1.0d0+ess)**2.0d0))**(gam/(1.0d0-gam))
	            if (cyc<tol) then
	                eqyf = 0.0d0
	            else	    
	                call newton_sf(eqyf,erryf,tol,ca,thes,ess)
	                if (erryf.eq.1) then
	                    eqyf = brent_yf(tol,ca,thes,ess)
	                endif
	            endif
	            ! (2-5) solve Y_a* given S*
	            cyc = ess/cf-((1.0d0-thes)/((1.0d0+ess)**2.0d0))**(gam/(1.0d0-gam))
	            if (cyc<tol) then
	                eqya = 0.0d0
	            else	            
                    call newton_sa(eqya,errya,tol,cf,thes,ess)
                    if (errya.eq.1) then
                        eqya = brent_ya(tol,cf,thes,ess)
                    endif
                endif                
            endif
        endif
        
		! -----------------------------------------------        
        ! (3) Solve equilibrium enactment probability
        ! -----------------------------------------------
		eprob = (thes+eqyf)/(1.0d0+eqyf+eqya)
		
		! ------------------------------------------------
		! (4) Solve equilibrium spending and payoff
		! ------------------------------------------------
		do ii = 1,nr
		     entrycost = ect
		    if (entrycost< 0.0d0) then
		        entrycost = 0.0d0
		    end if		    
			if ((pes(ii).eq.1).and.(ves(ii)>0.0d0)) then
				spd(ii) = ((bf*gam*ves(ii)*(1.0d0-thes+eqya)/((1+eqyf+eqya)**2.0d0))**(1.0d0/(1.0d0-gam)))+entrycost
				payoff(ii) = eprob*ves(ii)-spd(ii)
			elseif ((pes(ii).eq.1).and.(ves(ii)<0.0d0)) then
				spd(ii) = ((ba*gam*(-ves(ii))*(thes+eqyf)/((1+eqyf+eqya)**2.0d0))**(1.0d0/(1.0d0-gam)))+entrycost
				payoff(ii) = eprob*ves(ii)-spd(ii)						        
			else
				spd(ii) = 0.0d0
				payoff(ii) = eprob*ves(ii)
			end if
		end do
		
	end subroutine sim_esearch

    subroutine newton_fa(eqyn,errn,toln,cfn,can,thesn)
	    implicit none
	    ! Solves a system of two equations for Y_f* and Y_a* using Newton
	    ! Jacobian is analytically solved.
	    
	    real(8), intent(in):: toln, cfn, can, thesn
	    real(8), intent(out):: eqyn(2)
	    integer, intent(out):: errn	    
	    integer, parameter :: imax = 500
	    real(8) :: dfx(2,2), dx(2), fx(2), x0(2), df
	    integer :: iter
	    
        x0(1) = cfn
        x0(2) = can
        call faequ(fx,x0,cfn,can,thesn)	   
	     
	    errn = 0
	    iter = 0
	    df = (fx(1)**2.0d0+fx(2)**2.0d0)**0.5d0 
	    
	    do while ((df>toln).and.(errn.eq.0))
		    call fajac(dfx,errn,x0,cfn,can,thesn)		    
		    if (iter>imax) then
			    errn = 1
		    endif
		    if (errn.eq.0) then
		        dx(1) = dfx(1,1)*fx(1)+dfx(1,2)*fx(2)
		        dx(2) = dfx(2,1)*fx(1)+dfx(2,2)*fx(2)
		        x0(1) = x0(1)-dx(1)
		        x0(2) = x0(2)-dx(2)
		        if (x0(1)<toln) then
		            x0(1) = (toln+abs(x0(1)))*0.5d0
		        endif
		        if (x0(2)<toln) then
		            x0(2) = (toln+abs(x0(2)))*0.5d0
		        endif
		        call faequ(fx,x0,cfn,can,thesn)
		        df = (fx(1)**2.0d0+fx(2)**2.0d0)**0.5d0 		        
		    endif   
		    iter = iter + 1
	    end do
	    eqyn = x0	        
    end subroutine newton_fa

	subroutine faequ(fay,x0fa,cffa,cafa,thesfa)
		use globvar
		implicit none
		! A system of two equations for Y_f* and Y_a*
		
		real(8), intent(in) :: x0fa(2),cffa,cafa,thesfa
		real(8), intent(out):: fay(2)
		real(8) :: yf, ya		
		yf = x0fa(1)
		ya = x0fa(2)
		fay(1) = yf-cffa*(((1.0d0-thesfa+ya)/((1+yf+ya)**2.0d0))**(gam/(1.0d0-gam)))
		fay(2) = ya-cafa*(((thesfa+yf)/((1+yf+ya)**2.0d0))**(gam/(1.0d0-gam)))		
	end subroutine faequ	
	
	subroutine fajac(dfx_inv,errj,x0j,cfj,caj,thesj)
	    use globvar
	    implicit none
	    ! Jacobian inverse of the system of two equations for Y_f* and Y_a*
	    
	    real(8), intent(in) :: x0j(2), cfj, caj, thesj
	    real(8), intent(out) :: dfx_inv(2,2)
	    integer, intent(out) :: errj
	    real(8) :: yfj, yaj, jc1, jc2, dfxj(2,2), ckj 
	    
	    yfj = x0j(1)
	    yaj = x0j(2)
	   
        ! Jacobian	   
	    jc1 = (cfj*gam/(1.0d0-gam))*(1.0d0-thesj+yaj)**((2.0d0*gam-1.0d0)/(1.0d0-gam))
	    jc1 = jc1/((1.0d0+yfj+yaj)**((2.0d0*gam)/(1.0d0-gam)))
	    jc2 = (caj*gam/(1.0d0-gam))*(thesj+yfj)**((2.0d0*gam-1.0d0)/(1.0d0-gam))
	    jc2 = jc1/((1.0d0+yfj+yaj)**((2.0d0*gam)/(1.0d0-gam)))	    
	    dfxj(1,1) = 1.0d0+2.0d0*jc1*(1.0d0-thesj+yaj)/(1.0d0+yfj+yaj)
	    dfxj(1,2) = (-cfj)*jc1*(1.0d0-2.0d0*(1.0d0-thesj+yaj)/(1.0d0+yfj+yaj))
	    dfxj(2,1) = (-caj)*jc2*(1.0d0-2.0d0*(thesj+yfj)/(1.0d0+yfj+yaj))
	    dfxj(2,2) = 1.0d0+2.0d0*jc2*(thesj+yfj)/(1.0d0+yfj+yaj)	    
	    
	    ! Jacobian inverse
	    ckj = dfxj(1,1)*dfxj(2,2)-dfxj(1,2)*dfxj(2,1)
	    if (ckj.eq.0.0d0) then
	        errj = 1
	        dfx_inv = dfxj
	    else
	        errj = 0
	        dfx_inv(1,1) = (1.0d0/ckj)*dfxj(2,2)
	        dfx_inv(1,2) = -(1.0d0/ckj)*dfxj(1,2)
	        dfx_inv(2,1) = -(1.0d0/ckj)*dfxj(2,1)
	        dfx_inv(2,2) = (1.0d0/ckj)*dfxj(1,1)
	    endif
	end subroutine fajac

    subroutine newton_ff(eqyff,errf,tolf,conf,thesf)
	    implicit none	    
	    ! Solves an equation for Y_f* given that Y_a*=0 using Newton
	    ! Jacobian is analytically solved.
	    
	    real(8), intent(in):: tolf,conf,thesf
	    real(8), intent(out):: eqyff
	    integer, intent(out):: errf	    
	    integer, parameter :: imax = 30
	    real(8) :: dfxff, fxff, x0ff
	    integer :: iter
	    
        x0ff = conf
        fxff = x0ff-conf*(((1.0d0-thesf)/((1.0d0+x0ff)**2.0d0))**(gam/(1.0d0-gam)))
	    errf = 0
	    iter = 0
	    do while ((abs(fxff)>tolf).and.(errf.eq.0))
	        dfxff = ((1.0d0-thesf)**(gam/(1.0d0-gam)))/((1.0d0+x0ff)**((1.0d0+gam)/(1.0d0-gam)))
               dfxff = 1.0d0+2.0d0*conf*(gam/(1.0d0-gam))*dfxff		
		    if ((iter>imax).or.(dfxff.eq.0.0d0)) then
			    errf = 1
		    else
		        x0ff = x0ff-fxff/dfxff	
		        if (x0ff<tolf) then
		            x0ff = (abs(x0ff)+tolf)*0.5d0
		        endif		        
                fxff = x0ff-conf*(((1.0d0-thesf)/((1.0d0+x0ff)**2.0d0))**(gam/(1.0d0-gam)))
		        iter = iter + 1
		    endif
	    end do
	    eqyff = x0ff	    
    end subroutine newton_ff

    subroutine newton_aa(eqyaa,erra,tola,cona,thesa)
	    implicit none
	    ! Solves an equation for Y_a* given that Y_a*=0 using Newton
	    ! Jacobian is analytically solved.
	    
	    real(8), intent(in):: tola,cona,thesa
	    real(8), intent(out):: eqyaa
	    integer, intent(out):: erra	    
	    integer, parameter :: imax = 30	
	    real(8) :: dfxaa, fxaa, x0aa
	    integer :: iter
	    
        x0aa = cona
        fxaa = x0aa-cona*((thesa/((1.0d0+x0aa)**2.0d0))**(gam/(1.0d0-gam)))
	    erra = 0
	    iter = 0
	    do while ((abs(fxaa)>tola).and.(erra.eq.0))
	        dfxaa = (thesa**(gam/(1.0d0-gam)))/((1.0d0+x0aa)**((1.0d0+gam)/(1.0d0-gam)))	
               dfxaa = 1.0d0+2.0d0*cona*(gam/(1.0d0-gam))*dfxaa		            
		    if ((iter>imax).or.(dfxaa.eq.0.0d0)) then
			    erra = 1
		    else
		        x0aa = x0aa-fxaa/dfxaa
		        if (x0aa<tola) then
		            x0aa = (abs(x0aa)+tola)*0.5d0
		        endif   
                fxaa = x0aa-cona*((thesa/((1.0d0+x0aa)**2.0d0))**(gam/(1.0d0-gam)))
		        iter = iter + 1
		    endif
	    end do
	    eqyaa = x0aa    
    end subroutine newton_aa

    subroutine newton_s0(ess0,ers0,tol,css0)
	    implicit none
	    ! Solves an equation for Y_a* given that Y_a*=0 using Newton
	    ! Jacobian is analytically solved.
	    
	    real(8), intent(in):: tol,css0
	    real(8), intent(out):: ess0
	    integer, intent(out):: ers0
	    integer, parameter :: imax = 30	
	    real(8) :: dfx, fx, x0
	    integer :: iter
	    
        x0 = min(css0**((1.0d0-gam)/gam),css0/2.0d0)
        fx = x0*((1.0d0+x0)**(2.0d0*gam/(1.0d0-gam)))-css0    
        
	    ers0 = 0
	    iter = 0
	    do while ((abs(fx)>tol).and.(ers0.eq.0))
	        dfx = (1.0d0+x0)**(2.0d0*gam/(1.0d0-gam))
               dfx = dfx +(2.0d0*gam/(1.0d0-gam))*x0*((1.0d0+x0)**((3.0d0*gam-1.0d0)/(1.0d0-gam))) 	            
		    if ((iter>imax).or.(dfx.eq.0.0d0)) then
			    ers0 = 1
		    else 
		        x0 = x0-fx/dfx
		        if (x0<tol) then
		            x0 = (abs(x0)+tol)*0.5d0
		        endif   
                fx = x0*((1.0d0+x0)**(2.0d0*gam/(1.0d0-gam)))-css0
		        iter = iter + 1
		    endif
	    end do
	    ess0 = x0    
    end subroutine newton_s0

    subroutine newton_sf(ynsf,ersf,tosf,cosf,thsf,essf)
	    implicit none
	    ! Solves an equation for Y_f* given that S* using Newton
	    ! Jacobian is analytically solved.

	    real(8), intent(in):: tosf,cosf,thsf,essf  
	    real(8), intent(out):: ynsf
	    integer, intent(out):: ersf
	    integer, parameter :: imax = 30	
	    real(8) :: dfxsf, fxsf, x0sf, gc
	    integer :: iter
	    
        x0sf = cosf
        gc = gam/(1.0d0-gam)
        fxsf = essf-x0sf-cosf*(((thsf+x0sf)/((1.0d0+essf)**2.0d0))**gc)
	    ersf = 0
	    iter = 0
	    do while ((abs(fxsf)>tosf).and.(ersf.eq.0))
            dfxsf = -1.0d0 - cosf*gc*(((thsf+x0sf)/((1.0d0+essf)**2.0d0))**(gc-1.0d0))/((1.0d0+essf)**2.0d0)          
		    if ((iter>imax).or.(dfxsf.eq.0.0d0)) then
			    ersf = 1
		    else
		        x0sf = x0sf-fxsf/dfxsf
		        if (x0sf<tosf) then
		            x0sf = (abs(x0sf)+tosf)*.05d0
		        endif   
                fxsf = essf-x0sf-cosf*(((thsf+x0sf)/((1.0d0+essf)**2.0d0))**gc)
		        iter = iter + 1
		    endif
	    end do
	    ynsf = x0sf	    
    end subroutine newton_sf

    subroutine newton_sa(ynsa,ersa,tosa,cosa,thsa,essa)
	    implicit none
	    ! Solves an equation for Y_a* given that S* using Newton
	    ! Jacobian is analytically solved.

	    real(8), intent(in):: tosa,cosa,thsa,essa   
	    real(8), intent(out):: ynsa
	    integer, intent(out):: ersa 
	    integer, parameter :: imax = 30	
	    real(8) :: dfxsa, fxsa, x0sa, gc
	    integer :: iter
	    
        x0sa = cosa
        gc = gam/(1.0d0-gam)
        fxsa = essa-x0sa-cosa*(((1.0d0-thsa+x0sa)/((1.0d0+essa)**2.0d0))**gc)         
	    ersa = 0
	    iter = 0
	    do while ((abs(fxsa)>tosa).and.(ersa.eq.0))
            dfxsa = -1.0d0 - cosa*gc*(((thsa+x0sa)/((1.0d0+essa)**2.0d0))**(gc-1.0d0))/((1.0d0+essa)**2.0d0)
		    if ((iter>imax).or.(dfxsa.eq.0.0d0)) then
			    ersa = 1
		    else
		        x0sa = x0sa-fxsa/dfxsa
		        if (x0sa<tosa) then
		            x0sa = (abs(x0sa)+tosa)*0.5d0
		        endif  		         
                fxsa = essa-x0sa-cosa*(((1.0d0-thsa+x0sa)/((1.0d0+essa)**2.0d0))**gc)
		        iter = iter + 1
		    endif
	    end do
	    ynsa = x0sa   
    end subroutine newton_sa

    real(8) function brent_es(x1s,x2s,tol,cfs,cas,thess)
      
      ! Solves S* using Brent
      ! It is invoked if newton_fa doesn't work
      
        integer, parameter :: ITMAX=500       ! Maximum number of iterations
        real(8), parameter :: EPS=3.E-7       ! Machine floating-point precision
        real(8), intent(in) :: x1s, x2s, tol, cfs, cas, thess

        real(8) :: a, b, fa, fb
        integer :: iter
        real(8) :: c, fc,d, e
        real(8) :: s, p, q, r
        real(8) :: tol1, xm
        integer :: err
        
        A=x1s
        B=x2s
        FA=esequ(A,cfs,cas,thess)        
        FB=esequ(B,cfs,cas,thess)
        if(FB*FA.GT.0.) then
            print*, 'Root must be bracketed for brent_es.'
            stop
        end if
        FC=FB
        do  ITER=1,ITMAX
            if (FB*FC.GT.0.) then
                C=A
                FC=FA
                D=B-A
                E=D
            end if
            if (ABS(FC).LT.ABS(FB)) then
                A=B
                B=C
                C=A
                FA=FB
                FB=FC
                FC=FA
            end if
            TOL1=2.*EPS*ABS(B)+0.5*TOL
            XM=.5*(C-B)
            IF (ABS(XM).LE.TOL1 .OR. FB.EQ.0.)THEN
                brent_es=B
                RETURN
            ENDIF
            IF (ABS(E).GE.TOL1 .AND. ABS(FA).GT.ABS(FB)) THEN
                S=FB/FA
                IF (A.EQ.C) THEN
                    P=2.*XM*S
                    Q=1.-S
                ELSE
                    Q=FA/FC
                    R=FB/FC
                    P=S*(2.*XM*Q*(Q-R)-(B-A)*(R-1.))
                    Q=(Q-1.)*(R-1.)*(S-1.)
                ENDIF
                IF(P.GT.0.) Q=-Q
                P=ABS(P)
                IF(2.*P .LT. MIN(3.*XM*Q-ABS(TOL1*Q),ABS(E*Q))) THEN
                    E=D
                    D=P/Q
                ELSE
                    D=XM
                    E=D
                ENDIF
            ELSE
                D=XM
                E=D
            ENDIF
            A=B
            FA=FB
            IF (ABS(D) .GT. TOL1) THEN
                B=B+D
            ELSE
                B=B+SIGN(TOL1,XM)
            ENDIF
            FB=esequ(B,cfs,cas,thess)
        end do
        print*, 'BRENT_ES exceeding maximum iterations.'
        stop
        brent_es = B
        RETURN
    end function brent_es

	real(8) function esequ(ysq,cfsq,casq,thessq)
		use globvar
		implicit none
		! equation for S* (for brent_es)
		
		real(8), intent(in):: ysq, cfsq, casq, thessq
		real(8), parameter :: tol = 1.0E-12
		real(8) :: x1sq, x2sq, y1sq, y2sq, eqyfsq, eqyasq, cy
		integer :: ersq1, ersq2 
		    
	    !(1) Solve Y_f* and Y_a*
	    
	    cy = ysq/casq-(thessq/((1.0d0+ysq)**2.0d0))**(gam/(1.0d0-gam))
	    if (cy<tol) then
	        eqyfsq = 0.0d0
	    else	    
	        call newton_sf(eqyfsq,ersq1,tol,casq,thessq,ysq)
	        if (ersq1.eq.1) then
	            eqyfsq = brent_yf(tol,casq,thessq,ysq)
	        endif
	    endif
	    
	    cy = ysq/cfsq-((1.0d0-thessq)/((1.0d0+ysq)**2.0d0))**(gam/(1.0d0-gam))
	    if (cy<tol) then
	        eqyasq = 0.0d0
	    else
	        call newton_sa(eqyasq,ersq2,tol,cfsq,thessq,ysq)
            if (ersq2.eq.1) then
                eqyasq = brent_ya(tol,cfsq,thessq,ysq)
            endif
        endif	    
	    
        ! (2) Equation Y_f*+Y_a*-S*
        esequ = eqyfsq+eqyasq-ysq
        
	end function esequ

   real(8) function brent_s0(tol,css)
      
      ! Solves an equation for Y_a* given that Y_f*=0 using Brent
      ! It is invoked when newton_aa doesn't work
      
        integer, parameter :: ITMAX=500       ! Maximum number of iterations
        real(8), parameter :: EPS=3.E-7       ! Machine floating-point precision
        real(8), intent(in) :: tol,css

        real(8) :: a, b, fa, fb
        integer :: iter
        real(8) :: c, fc,d, e
        real(8) :: s, p, q, r
        real(8) :: tol1, xm
        		
        A= 0.0d0
        B= css
        FA= A*((1.0d0+A)**(2.0d0*gam/(1.0d0-gam)))-css
        FB= B*((1.0d0+B)**(2.0d0*gam/(1.0d0-gam)))-css
        
        if(FB*FA.GT.0.) then
            print*, 'Root must be bracketed for brent_s0.'
            stop
        end if
        FC=FB
        do  ITER=1,ITMAX
            if (FB*FC.GT.0.) then
                C=A
                FC=FA
                D=B-A
                E=D
            end if
            if (ABS(FC).LT.ABS(FB)) then
                A=B
                B=C
                C=A
                FA=FB
                FB=FC
                FC=FA
            end if
            TOL1=2.*EPS*ABS(B)+0.5*TOL
            XM=.5*(C-B)
            IF (ABS(XM).LE.TOL1 .OR. FB.EQ.0.)THEN
                brent_s0=B
                RETURN
            ENDIF
            IF (ABS(E).GE.TOL1 .AND. ABS(FA).GT.ABS(FB)) THEN
                S=FB/FA
                IF (A.EQ.C) THEN
                    P=2.*XM*S
                    Q=1.-S
                ELSE
                    Q=FA/FC
                    R=FB/FC
                    P=S*(2.*XM*Q*(Q-R)-(B-A)*(R-1.))
                    Q=(Q-1.)*(R-1.)*(S-1.)
                ENDIF
                IF(P.GT.0.) Q=-Q
                P=ABS(P)
                IF(2.*P .LT. MIN(3.*XM*Q-ABS(TOL1*Q),ABS(E*Q))) THEN
                    E=D
                    D=P/Q
                ELSE
                    D=XM
                    E=D
                ENDIF
            ELSE
                D=XM
                E=D
            ENDIF
            A=B
            FA=FB
            IF (ABS(D) .GT. TOL1) THEN
                B=B+D
            ELSE
                B=B+SIGN(TOL1,XM)
            ENDIF
            FB= B*((1.0d0+B)**(2.0d0*gam/(1.0d0-gam)))-css
        end do
        print*, 'BRENT_S0 exceeding maximum iterations.'
        stop
        brent_s0 = B
        RETURN
    end function brent_s0

   real(8) function brent_ff(x1bff,x2bff,tol,cbff,thbff)
      
      ! Solves an equation for Y_f* given that Y_a*=0 using Brent
      ! It is invoked when newton_ff doesn't work
      
        integer, parameter :: ITMAX=500       ! Maximum number of iterations
        real(8), parameter :: EPS=3.E-7       ! Machine floating-point precision
        real(8), intent(in) :: x1bff, x2bff, tol, cbff, thbff

        real(8) :: a, b, fa, fb
        integer :: iter
        real(8) :: c, fc,d, e
        real(8) :: s, p, q, r
        real(8) :: tol1, xm
        		
        A=x1bff
        B=x2bff
        FA= A-cbff*(((1.0d0-thbff)/((1.0d0+A)**2.0d0))**(gam/(1.0d0-gam)))
        FB= B-cbff*(((1.0d0-thbff)/((1.0d0+B)**2.0d0))**(gam/(1.0d0-gam)))
        
        if(FB*FA.GT.0.) then
            print*, 'Root must be bracketed for brent_ff.'
            stop
        end if
        FC=FB
        do  ITER=1,ITMAX
            if (FB*FC.GT.0.) then
                C=A
                FC=FA
                D=B-A
                E=D
            end if
            if (ABS(FC).LT.ABS(FB)) then
                A=B
                B=C
                C=A
                FA=FB
                FB=FC
                FC=FA
            end if
            TOL1=2.*EPS*ABS(B)+0.5*TOL
            XM=.5*(C-B)
            IF (ABS(XM).LE.TOL1 .OR. FB.EQ.0.)THEN
                brent_ff=B
                RETURN
            ENDIF
            IF (ABS(E).GE.TOL1 .AND. ABS(FA).GT.ABS(FB)) THEN
                S=FB/FA
                IF (A.EQ.C) THEN
                    P=2.*XM*S
                    Q=1.-S
                ELSE
                    Q=FA/FC
                    R=FB/FC
                    P=S*(2.*XM*Q*(Q-R)-(B-A)*(R-1.))
                    Q=(Q-1.)*(R-1.)*(S-1.)
                ENDIF
                IF(P.GT.0.) Q=-Q
                P=ABS(P)
                IF(2.*P .LT. MIN(3.*XM*Q-ABS(TOL1*Q),ABS(E*Q))) THEN
                    E=D
                    D=P/Q
                ELSE
                    D=XM
                    E=D
                ENDIF
            ELSE
                D=XM
                E=D
            ENDIF
            A=B
            FA=FB
            IF (ABS(D) .GT. TOL1) THEN
                B=B+D
            ELSE
                B=B+SIGN(TOL1,XM)
            ENDIF
            FB=B-cbff*(((1.0d0-thbff)/((1.0d0+B)**2.0d0))**(gam/(1.0d0-gam)))
        end do
        print*, 'BRENT_FF exceeding maximum iterations.'
        stop
        brent_ff = B
        RETURN
    end function brent_ff

   real(8) function brent_aa(x1baa,x2baa,tol,cbaa,thbaa)
      
      ! Solves an equation for Y_a* given that Y_f*=0 using Brent
      ! It is invoked when newton_aa doesn't work
      
        integer, parameter :: ITMAX=500       ! Maximum number of iterations
        real(8), parameter :: EPS=3.E-7       ! Machine floating-point precision
        real(8), intent(in) :: x1baa, x2baa, tol, cbaa, thbaa

        real(8) :: a, b, fa, fb
        integer :: iter
        real(8) :: c, fc,d, e
        real(8) :: s, p, q, r
        real(8) :: tol1, xm
        		
        A=x1baa
        B=x2baa
        FA= A-cbaa*((thbaa/((1.0d0+A)**2.0d0))**(gam/(1.0d0-gam)))
        FB= B-cbaa*((thbaa/((1.0d0+B)**2.0d0))**(gam/(1.0d0-gam)))

        if(FB*FA.GT.0.) then
            print*, 'Root must be bracketed for brent_aa.'
            stop
        end if
        FC=FB
        do  ITER=1,ITMAX
            if (FB*FC.GT.0.) then
                C=A
                FC=FA
                D=B-A
                E=D
            end if
            if (ABS(FC).LT.ABS(FB)) then
                A=B
                B=C
                C=A
                FA=FB
                FB=FC
                FC=FA
            end if
            TOL1=2.*EPS*ABS(B)+0.5*TOL
            XM=.5*(C-B)
            IF (ABS(XM).LE.TOL1 .OR. FB.EQ.0.)THEN
                brent_aa=B
                RETURN
            ENDIF
            IF (ABS(E).GE.TOL1 .AND. ABS(FA).GT.ABS(FB)) THEN
                S=FB/FA
                IF (A.EQ.C) THEN
                    P=2.*XM*S
                    Q=1.-S
                ELSE
                    Q=FA/FC
                    R=FB/FC
                    P=S*(2.*XM*Q*(Q-R)-(B-A)*(R-1.))
                    Q=(Q-1.)*(R-1.)*(S-1.)
                ENDIF
                IF(P.GT.0.) Q=-Q
                P=ABS(P)
                IF(2.*P .LT. MIN(3.*XM*Q-ABS(TOL1*Q),ABS(E*Q))) THEN
                    E=D
                    D=P/Q
                ELSE
                    D=XM
                    E=D
                ENDIF
            ELSE
                D=XM
                E=D
            ENDIF
            A=B
            FA=FB
            IF (ABS(D) .GT. TOL1) THEN
                B=B+D
            ELSE
                B=B+SIGN(TOL1,XM)
            ENDIF
            FB=B-cbaa*((thbaa/((1.0d0+B)**2.0d0))**(gam/(1.0d0-gam)))
        end do
        print*, 'BRENT_AA exceeding maximum iterations.'
        stop
        brent_aa = B
        RETURN
    end function brent_aa

    real(8) function brent_yf(tol,conf,thesf,esbf)
      
      ! Solves an equation for Y_f* given that S* using Brent
      ! It is invoked when newton_sf doesn't work
      
        integer, parameter :: ITMAX=500       ! Maximum number of iterations
        real(8), parameter :: EPS=3.E-7       ! Machine floating-point precision
        real(8), intent(in) :: tol, conf, thesf, esbf

        real(8) :: a, b, fa, fb
        integer :: iter
        real(8) :: c, fc,d, e
        real(8) :: s, p, q, r
        real(8) :: tol1, xm
        
        A=0.0d0
        B=esbf
        FA= esbf-A-conf*(((thesf+A)/((1.0d0+esbf)**2.0d0))**(gam/(1.0d0-gam)))
        FB= esbf-B-conf*(((thesf+B)/((1.0d0+esbf)**2.0d0))**(gam/(1.0d0-gam)))        
        if(FB*FA.GT.0.) then
            print*, 'Root must be bracketed for brent_yf.'         
            stop
        end if
        FC=FB
        do  ITER=1,ITMAX
            if (FB*FC.GT.0.) then
                C=A
                FC=FA
                D=B-A
                E=D
            end if
            if (ABS(FC).LT.ABS(FB)) then
                A=B
                B=C
                C=A
                FA=FB
                FB=FC
                FC=FA
            end if
            TOL1=2.*EPS*ABS(B)+0.5*TOL
            XM=.5*(C-B)
            IF (ABS(XM).LE.TOL1 .OR. FB.EQ.0.)THEN
                brent_yf=B
                RETURN
            ENDIF
            IF (ABS(E).GE.TOL1 .AND. ABS(FA).GT.ABS(FB)) THEN
                S=FB/FA
                IF (A.EQ.C) THEN
                    P=2.*XM*S
                    Q=1.-S
                ELSE
                    Q=FA/FC
                    R=FB/FC
                    P=S*(2.*XM*Q*(Q-R)-(B-A)*(R-1.))
                    Q=(Q-1.)*(R-1.)*(S-1.)
                ENDIF
                IF(P.GT.0.) Q=-Q
                P=ABS(P)
                IF(2.*P .LT. MIN(3.*XM*Q-ABS(TOL1*Q),ABS(E*Q))) THEN
                    E=D
                    D=P/Q
                ELSE
                    D=XM
                    E=D
                ENDIF
            ELSE
                D=XM
                E=D
            ENDIF
            A=B
            FA=FB
            IF (ABS(D) .GT. TOL1) THEN
                B=B+D
            ELSE
                B=B+SIGN(TOL1,XM)
            ENDIF
            FB=esbf-B-conf*(((thesf+B)/((1.0d0+esbf)**2.0d0))**(gam/(1.0d0-gam)))
        end do
        print*, 'BRENT_YF exceeding maximum iterations.'
        stop
        brent_yf = B
        RETURN
    end function brent_yf
	
    real(8) function brent_ya(tol,cona,thesa,esba)
      
      ! Solves an equation for Y_f* given that S* using Brent
      ! It is invoked when newton_sa doesn't work
      
        integer, parameter :: ITMAX=500       ! Maximum number of iterations
        real(8), parameter :: EPS=3.E-7       ! Machine floating-point precision
        real(8), intent(in) :: tol, cona, thesa, esba

        real(8) :: a, b, fa, fb
        integer :: iter
        real(8) :: c, fc,d, e
        real(8) :: s, p, q, r
        real(8) :: tol1, xm
        
        A= 0.0d0
        B= esba
        FA= esba-A-cona*(((1.0d0-thesa+A)/((1.0d0+esba)**2.0d0))**(gam/(1.0d0-gam)))
        FB= esba-B-cona*(((1.0d0-thesa+B)/((1.0d0+esba)**2.0d0))**(gam/(1.0d0-gam)))        
        if(FB*FA.GT.0.) then
            print*, 'Root must be bracketed for brent_ya.'
            stop
        end if
        FC=FB
        do  ITER=1,ITMAX
            if (FB*FC.GT.0.) then
                C=A
                FC=FA
                D=B-A
                E=D
            end if
            if (ABS(FC).LT.ABS(FB)) then
                A=B
                B=C
                C=A
                FA=FB
                FB=FC
                FC=FA
            end if
            TOL1=2.*EPS*ABS(B)+0.5*TOL
            XM=.5*(C-B)
            IF (ABS(XM).LE.TOL1 .OR. FB.EQ.0.)THEN
                brent_ya=B
                RETURN
            ENDIF
            IF (ABS(E).GE.TOL1 .AND. ABS(FA).GT.ABS(FB)) THEN
                S=FB/FA
                IF (A.EQ.C) THEN
                    P=2.*XM*S
                    Q=1.-S
                ELSE
                    Q=FA/FC
                    R=FB/FC
                    P=S*(2.*XM*Q*(Q-R)-(B-A)*(R-1.))
                    Q=(Q-1.)*(R-1.)*(S-1.)
                ENDIF
                IF(P.GT.0.) Q=-Q
                P=ABS(P)
                IF(2.*P .LT. MIN(3.*XM*Q-ABS(TOL1*Q),ABS(E*Q))) THEN
                    E=D
                    D=P/Q
                ELSE
                    D=XM
                    E=D
                ENDIF
            ELSE
                D=XM
                E=D
            ENDIF
            A=B
            FA=FB
            IF (ABS(D) .GT. TOL1) THEN
                B=B+D
            ELSE
                B=B+SIGN(TOL1,XM)
            ENDIF
            FB= esba-B-cona*(((1.0d0-thesa+B)/((1.0d0+esba)**2.0d0))**(gam/(1.0d0-gam)))
        end do
        print*, 'BRENT_YA exceeding maximum iterations.'
        stop
        brent_ya = B
        RETURN
    end function brent_ya
	
end module esolver