module etoolbox
use globvar

! Contains various subroutines used for calculating the likelihood.

implicit none

contains

    subroutine parscale(par_n, par)
        implicit none
        integer, intent(in) :: par_n
        real(8), intent(in) :: par(par_n)
        integer:: i

        bf = par(1)*scale_par(1)
        ba = par(2)*scale_par(2)                     
        gam = par(3)*scale_par(3)
	 varin = par(4)*scale_par(4)
	
        do i = 1,6
            del(i) = par(i+4)*scale_par(i+4)
        end do
            
        do i = 1,8 
            alp(i) = par(i+10)*scale_par(i+10)
        end do
        
        do i = 1,4
            varvu(i) = par(i+18)*scale_par(i+18)
        end do 
              
    end subroutine parscale 
    
    subroutine parscale_inv(par)
        implicit none
        real(8), intent(out) :: par(npar)
        integer:: i
                                
        par(1) = bf
        par(2) = ba
        par(3) = gam
        par(4) = varin
        do i = 1,6
            par(i+4) = del(i)
        end do 
        do i = 1,8
            par(i+10) = alp(i)
        end do       
        do i = 1,4
            par(i+18) = varvu(i)
        end do 
              
    end subroutine parscale_inv

    subroutine parscale_avar(par_n, par)
        implicit none
        integer, intent(in) :: par_n
        real(8), intent(in) :: par(par_n)
        integer:: i
                                
        bf = par(1)
        ba = par(2)
        gam = par(3)
        varin = par(4)
        do i = 1,6
            del(i) = par(i+4)
        end do
        do i = 1,8
            alp(i) = par(i+10)
        end do 
        do i = 1,4
            varvu(i) = par(i+18)
        end do       
              
    end subroutine parscale_avar
	
	SUBROUTINE findinv(matrix, inverse, n, errorflag)
	IMPLICIT NONE
	!Declarations
	INTEGER, INTENT(IN) :: n
	INTEGER, INTENT(OUT) :: errorflag  !Return error status. -1 for error, 0 for normal
	REAL(8), INTENT(IN), DIMENSION(n,n) :: matrix  !Input matrix
	REAL(8), INTENT(OUT), DIMENSION(n,n) :: inverse !Inverted matrix
	
	LOGICAL :: FLAG = .TRUE.
	INTEGER :: i, j, k, l
	REAL(8) :: m
	REAL(8), DIMENSION(n,2*n) :: augmatrix !augmented matrix
	
	!Augment input matrix with an identity matrix
	DO i = 1, n
		DO j = 1, 2*n
			IF (j <= n ) THEN
				augmatrix(i,j) = matrix(i,j)
			ELSE IF ((i+n) == j) THEN
				augmatrix(i,j) = 1
			Else
				augmatrix(i,j) = 0
			ENDIF
		END DO
	END DO
	
	!Reduce augmented matrix to upper traingular form
	DO k =1, n-1
		IF (augmatrix(k,k) == 0) THEN
			FLAG = .FALSE.
			DO i = k+1, n
				IF (augmatrix(i,k) /= 0) THEN
					DO j = 1,2*n
						augmatrix(k,j) = augmatrix(k,j)+augmatrix(i,j)
					END DO
					FLAG = .TRUE.
					EXIT
				ENDIF
				IF (FLAG .EQV. .FALSE.) THEN
					PRINT*, "Matrix is non - invertible"
					inverse = 0
					errorflag = -1
					return
				ENDIF
			END DO
		ENDIF
		DO j = k+1, n			
			m = augmatrix(j,k)/augmatrix(k,k)
			DO i = k, 2*n
				augmatrix(j,i) = augmatrix(j,i) - m*augmatrix(k,i)
			END DO
		END DO
	END DO
	
	!Test for invertibility
	DO i = 1, n
		IF (augmatrix(i,i) == 0) THEN
			PRINT*, "Matrix is non - invertible"
			inverse = 0
			errorflag = -1
			return
		ENDIF
	END DO
	
	!Make diagonal elements as 1
	DO i = 1 , n
		m = augmatrix(i,i)
		DO j = i , (2 * n)				
			   augmatrix(i,j) = (augmatrix(i,j) / m)
		END DO
	END DO
	
	!Reduced right side half of augmented matrix to identity matrix
	DO k = n-1, 1, -1
		DO i =1, k
		m = augmatrix(i,k+1)
			DO j = k, (2*n)
				augmatrix(i,j) = augmatrix(i,j) -augmatrix(k+1,j) * m
			END DO
		END DO
	END DO				
	
	!store answer
	DO i =1, n
		DO j = 1, n
			inverse(i,j) = augmatrix(i,j+n)
		END DO
	END DO
	errorflag = 0
    END SUBROUTINE findinv

    real(8) function uniform()

!   Returns a unit pseudorandom R8.
!   Parameters:
!   Input/output, integer SEED, the "seed" value, which should NOT be 0. On output, SEED has been updated.
!   Output, real (8), a new pseudorandom variate, strictly between 0 and 1.
!   Karam: this code seems to create discrepancy in the results using debug vs. release.
!
    implicit none

    integer(4), parameter :: i4_huge = 2147483647
    integer(4):: k

    if ( seed == 0 ) then
        write ( *, '(a)' ) ' '
        write ( *, '(a)' ) 'R8_UNIFORM_01 - Fatal error!'
        write ( *, '(a)' ) '  Input value of SEED = 0.'
        stop
    end if

    k = seed / 127773

    seed = 16807 * ( seed - k * 127773 ) - k * 2836

    if ( seed < 0 ) then
        seed = seed + i4_huge
    end if

    uniform = real(seed) * 4.656612875D-10

    end function uniform

    real(8) function al_norm(xan,upper)
    !Computes the cumulative density of the standard normal distribution.
    !Input, real(8) X, is one endpoint of the semi-infinite interval over which the integration takes place.
    !Input, logical UPPER, determines whether the upper or lower interval is to be integrated:
    !    .TRUE.  => integrate from X to + Infinity;
    !    .FALSE. => integrate from - Infinity to X.
    !Output, real(8) ALNORM, the integral of the standard normal distribution over the desired interval.
    implicit none

    real(8), intent(in) :: xan
    real(8), parameter :: a1 = 5.75885480458D+00
    real(8), parameter :: a2 = 2.62433121679D+00
    real(8), parameter :: a3 = 5.92885724438D+00
    real(8), parameter :: b1 = -29.8213557807D+00
    real(8), parameter :: b2 = 48.6959930692D+00
    real(8), parameter :: c1 = -0.000000038052D+00
    real(8), parameter :: c2 = 0.000398064794D+00
    real(8), parameter :: c3 = -0.151679116635D+00
    real(8), parameter :: c4 = 4.8385912808D+00
    real(8), parameter :: c5 = 0.742380924027D+00
    real(8), parameter :: c6 = 3.99019417011D+00
    real(8), parameter :: con = 1.28D+00
    real(8), parameter :: d1 = 1.00000615302D+00
    real(8), parameter :: d2 = 1.98615381364D+00
    real(8), parameter :: d3 = 5.29330324926D+00
    real(8), parameter :: d4 = -15.1508972451D+00
    real(8), parameter :: d5 = 30.789933034D+00
    real(8), parameter :: ltone = 7.0D+00
    real(8), parameter :: p = 0.398942280444D+00
    real(8), parameter :: q = 0.39990348504D+00
    real(8), parameter :: r = 0.398942280385D+00
    logical up
    logical upper
    real(8), parameter :: utzero = 18.66D+00
    real(8) :: yan, zan

    up = upper
    zan = xan
    if ( zan < 0.0D+00 ) then
    up = .not. up
    zan = - zan
    end if
    if ( ltone < zan .and. ( ( .not. up ) .or. utzero < zan ) ) then
    if ( up ) then
      al_norm = 0.0D+00
    else
      al_norm = 1.0D+00
    end if
    return
    end if
    yan = 0.5D+00 * zan * zan
    if ( zan <= con ) then
    al_norm = 0.5D+00 - zan * ( p - q * yan &
      / ( yan + a1 + b1 &
      / ( yan + a2 + b2 &
      / ( yan + a3 ))))
    else
    al_norm = r * exp ( - yan ) &
      / ( zan + c1 + d1 &
      / ( zan + c2 + d2 &
      / ( zan + c3 + d3 &
      / ( zan + c4 + d4 &
      / ( zan + c5 + d5 &
      / ( zan + c6 ))))))
    end if
    if ( .not. up ) then
    al_norm = 1.0D+00 - al_norm
    end if
    return
    end function al_norm


    subroutine piksrt(arrn,array)
    implicit none
    integer, intent(in) :: arrn
    real(8), intent(inout) :: array(arrn)
    real(8):: a
    integer :: i, j
    
    do j=2,arrn
        a = array(j)
        do i=j-1,1,-1
            if (array(i)<=a) goto 10
            array(i+1)=array(i)
        end do
	    i=0
10      array(i+1)=a
    end do
    return
    end subroutine piksrt


    subroutine choldc(chn, carr)
    implicit none
    integer, intent(in) :: chn
    real(8), intent(inout) :: carr(chn,chn)
    integer :: ii, jj, kk
    real(8) :: summ, chp(chn)
    
    ! input: carr(chn,chn) is a symmetric and positive definite matrix.
    ! outout: carr(chn,chn) is a lower triangular matrix L, so that L*L' equals the original matrix.
    
    do ii = 1,chn
        do jj = ii, chn
            summ = carr(ii,jj)
            do kk= (ii-1),1,-1
                summ = summ - carr(ii,kk)*carr(jj,kk)
            end do
            if (ii.eq.jj) then
                if (summ.le.0.0d0) then
                     print *, "choldc failed"
                     return
                else
                	chp(ii) = sqrt(summ)
                endif
            else
                carr(jj,ii) = summ/chp(ii)
            endif
        enddo
    enddo

    do ii = 1,chn
        do jj = 1,chn
            if (ii<jj) then
                carr(ii,jj) = 0.0d0
            elseif (ii.eq.jj) then
                carr(ii,jj) = chp(ii)
            endif
        end do
    end do    
    
    return    
    end subroutine choldc
	
end module etoolbox
