subroutine lhood_sim(par_n, par, lvalue)

use globvar
use etoolbox
use esolver

implicit none

integer, intent(in) :: par_n
real(8), intent(in) :: par(par_n)
real(8), intent(out) :: lvalue

real(8), parameter :: toll = 1.0E-16
real(8), parameter :: penalty = -log(toll)

integer :: ss, kk, gg, ii, jj, enum, jjn, dev, mdx
real(8) :: iz, thes, lcon, util0, util1, lck 

real(8), dimension(nr) :: v, tspd, lspd
integer, dimension(num) :: equinx
real(8), dimension(num):: equthn, equsum
real(8), dimension(num,nr) :: spend, payoff

real(8):: varpos, ulvalue
real(8):: cspd(nr), lcspd(nr), cnum

	lvalue = 0.0d0
	do jj = 1,nr
	    tspd(jj) = 0.0d0
	    cspd(jj) = 0.0d0
	end do
		
	call parscale(par_n,par)
	
	!(1) Check the conditions for parameters
	
	varpos = 0.0d0
	if (bf.le.0.0d0) then
	    varpos = varpos-bf
	end if
	if (gam.ge.1.0d0) then
	    varpos = varpos +(1.0d0-gam)
	end if
	if (gam.le.0.0d0) then
	    varpos = varpos-gam
	end if
	if (ect.le.0.0d0) then
	    varpos = varpos -ect
	end if
	if (varin<0.0d0) then
	    varpos = varpos -varin
	end if
	do jj = 1,nr
	    if (varvu(jj)<0.0d0) then
	        varpos = varpos - varvu(jj)
	    end if
	end do
	 
	if (varpos == 0.0d0) then

	!(2) Calculate the simulated likelihood
	
	    do gg = 1,np
		    lcon = 0.0d0
			do jj = 1,nr
				lspd(jj) = 0.0d0
				lcspd(jj) = 0.0d0
			end do
			cnum = 0.0d0
		    do ss = 1,ns
		        kk = (gg-1)*ns+ss
	    !(2-1) Generate theta's
			    iz = 0.0d0
			    do ii = 1,6
				    iz = iz + Z(gg,ii)*del(ii)
			    end do			    
			    thes = iz+varin*simshock(kk,1)		    
	    !(2-2) Generate values
			    do jj = 1,nr
				    if (W(gg,jj).eq.1) then
					    v(jj) =exp(alp(jj)+alp(jj+nr)*X(gg,jj)+simshock(kk,(jj+1))*varvu(jj))
				    else
					    v(jj) =-exp(alp(jj)+alp(jj+nr)*X(gg,jj)+simshock(kk,(jj+1))*varvu(jj))
				    end if
			    end do
		!(2-3) Calculate equilibrium in each participation case
			    do ii = 1,num
				    call sim_esearch(v,ptn(ii,:),thes,equthn(ii),payoff(ii,:),spend(ii,:))
			    end do
		!(2-4) Find equilibria by checking a deviation 
			    enum = 0
			    do ii = 1,num
			        jj = 1
				    dev = 0
				    equinx(ii) = 0
				    do while((jj<=nr).and.(dev.eq.0))
				        jjn = int(2.0d0**((nr-jj)*1.0d0))
				        util0 = payoff(ii,jj)			    
			            if (ptn(ii,jj)==0) then
			                util1 = payoff((ii+jjn),jj)
			            else
			                util1 = payoff((ii-jjn),jj)
			            end if
					    if (util1>util0) then
						    dev = 1
					    end if
					    jj = jj+1
				    end do
				    if (dev.eq.0) then
                        enum = enum+1
                        equinx(enum) = ii
                        equsum(enum) = 0.0d0
                        do jj = 1,nr
                            equsum(enum) = equsum(enum)+payoff(ii,jj)
                        end do                    
				    end if
                end do
		!(2-5) Find an equilibrium that is pareto optimal and check if the equilibrium is consistent
	    !      with the observed entry profile.		    
			    if (enum>0) then
                    mdx = equinx(maxloc(equsum(1:enum),1))
                    do jj = 1,nr
                        lspd(jj) = lspd(jj) + spend(mdx,jj)/(ns*1.0d0)
			        end do
		            ! lck is 0 if the selected equilibrium is equal to the observed equilibrium                
                    lck = 0.0d0
                    do jj = 1,nr
                        lck = lck+((ptn(mdx,jj)-rst_ptn(gg,jj))*1.0d0)**2.0d0
                    end do
	    !(2-6) Calculate the likelihood
			        if (lck.eq.0.0d0) then
			            if (rst_psg(gg)==1) then
				            lcon = lcon + equthn(mdx)/(ns*1.0d0)
				        else
				            lcon = lcon + (1.0d0-equthn(mdx))/(ns*1.0d0)
				        end if
				        cnum = cnum + 1.0d0
				        do jj = 1,nr
				            lcspd(jj) = lcspd(jj) + spend(mdx,jj)
				        end do			                             
                    end if
			    end if    
		    end do
		    
		    if (lcon<toll) then
			    lvalue = lvalue - penalty
		    else
			    lvalue = lvalue + log(lcon)
		    end if
			
			do jj = 1,nr
				tspd(jj) = tspd(jj) + lspd(jj)
			end do
			
			if (cnum>0.0d0) then
			    do jj = 1,nr
			        cspd(jj) = cspd(jj) + lcspd(jj)/cnum
			    end do
			end if
    		
	    end do
    	
	    lvalue = lvalue/(np*1.0d0)
	    ulvalue = lvalue	    

	    ! (3) add penalty (moment constraints)

        lvalue = ulvalue
	    do jj = 1,nr
	        lvalue = lvalue - weight*((1.0d0 -(tspd(jj)/rst_spd(jj)))**2.0d0)     
	    end do
	
	! this code is to write on a separate file. (unnecessary for appspack)
	    print *, "value:", ulvalue, lvalue
	    write(1000,*), "value:", ulvalue, lvalue
	    do jj = 1,nr
	        print *, rst_spd(jj)/unitm, tspd(jj)/unitm, cspd(jj)/unitm
	    end do
        
        ! this weight code is for calculating asymptotic variance
        
        do jj = 1,nr
            lagr(jj) = (2.0d0*np)*weight/tspd(jj)
        end do	    	    
	    
    else
    	lvalue = -penalty-varpos   
	end if
		
	lvalue = -lvalue
	
end subroutine lhood_sim
