program main_moments

! This program reads the parameters and the lower triangluar matrix of the 
! asymptotic variance-covariance matrix, and perform Monte-Carlo simulation
! return the simulated moments and their confidence intervals.

use globvar
use dataread
use etoolbox
use csv_file

implicit none

include "mpif.h"

integer :: i, par_n, j, upn, lrn
real(8) :: para(npar), lval
real(8) :: start, finish
integer :: node_id, ierr, ecode

real(8), parameter :: level = 0.05d0
integer, parameter :: nmom = 56
integer, parameter :: msize = nmom*nci
real(8) :: mommtx(nci,nmom), momvec(msize), tmomvec(msize) 
real(8) :: rmomvec(msize), vec(nci), mom0(nmom)
real(8) :: par1(npar), stdmat(npar,npar)

integer :: ii, jj, kk, ss, vi, vj


!**************
!(1) Read Data
!**************

    call MPI_INIT(ierr)
    call MPI_COMM_RANK(MPI_COMM_WORLD, node_id, ierr) 	    !0 to nproc-1
    call MPI_COMM_SIZE(MPI_COMM_WORLD, nproc, ierr) 	    !total number of nodes

    call datareadroutine

!***********************
!(2) Initial Parameters
!***********************  

    open(1001,file="est_par.txt")
    read (1001,*)
    read (1001,*), par_n
    do i=1,npar
         read (1001,*), para(i)
    end do
    close(1001)

    ! scaling
    do i = 1,npar
        scale_par(i) = 1.0d0
    end do
    scale_par(1)  = 0.0001d0
    scale_par(2)  = 0.0001d0
    scale_par(11) = 10.0d0
    scale_par(12) = 10.0d0
    scale_par(13) = 10.0d0
    scale_par(14) = 10.0d0
     

    ! confidence interval parameters
    upn = floor(nci*(1.0d0-level*0.5d0))
    lrn = floor(nci*level*0.50d0)  
    call parscale(npar,para)
    call parscale_inv(para)
    call moments(npar, para, nmom, mom0)

    ! read the variance-covariance matrix
    open(5000, file ="stdmat.txt")
    do vi = 1,par_n
        do vj = 1,par_n
            read (5000,*), stdmat(vi,vj)
        end do
    end do
    close(5000)

    call cpu_time(start)
!**************************
!(3) Calculate the Moments 
!**************************
     ii = node_id+1
     jj = 0
     do vi = 1,msize
        tmomvec(vi) = 0.0d0
     end do
     do while (ii.le.nci)
        ! (1) obtain a vector of parameters given shocks and stdmat
        do vi = 1,par_n
            par1(vi) = 0.0d0
            do vj = 1,par_n
                par1(vi) = par1(vi)+ parshock(ii,vj)*stdmat(vi,vj)
            end do
            par1(vi) = par1(vi)+para(vi)
        end do
 
        ! (2) given the parameter vector, obtain the moments
        do kk = 1,msize
            momvec(kk) = 0.0d0 
        end do
        call moments(par_n, par1, nmom, momvec((ii-1)*nmom+1:ii*nmom))
        do vi = 1,msize
 	     tmomvec(vi) =  tmomvec(vi)+ momvec(vi)
        end do 
        ! (3) update ii and jj
        jj = jj + 1
        ii = nproc*jj+node_id+1        
    end do
    call MPI_REDUCE(tmomvec,rmomvec,msize,MPI_DOUBLE_PRECISION,MPI_SUM,main_node,MPI_COMM_WORLD,ierr) 
 

!**************************
!(4) Calculate the CI 
!**************************

    if (node_id.eq.main_node) then
        do ii = 1,nci
            mommtx(ii,:) = rmomvec((nmom*(ii-1))+1:nmom*ii)
        end do 

        ! For each moment, sort the simulated moments (ascending) 
 
        print *, "Confidence Interval for Moments"
        print *, "Level:", level
        open(6000, file ="moments.txt")
        do vi = 1,nmom-1
            do ss= 1,nci
                vec(ss) = mommtx(ss,vi)
            end do
            call piksrt(nci,vec)
            ! Obtain confidence intervals
            write(6000,*) mom0(vi), vec(lrn), vec(upn)
            end do
        close(6000)
	 print *, "Number of Parameters that went out of bounds:", sum(mommtx(:,nmom))

    end if
    
    call cpu_time(finish)
    print '("Time = ",f15.3," seconds.")',finish-start
 
    call MPI_FINALIZE(ierr)       
end program main_moments